@@ -1,14 +1,14 @@
 /*
-   cfftf.c
-   Swarztrauber complex FFT routines (forward direction).
-
-   $Id$
+ * $Id: cfftf.c,v 1.1.1.1 2005/09/18 22:04:52 dhmunro Exp $
+ * Swarztrauber complex FFT routines (forward direction).
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
-
-
-extern void cfftf (long n,double c[],double wsave[]);
-extern void cfftf1 (long n,double c[],double ch[],double wa[],long ifac[]);
+#include "cfft.h"
 
 static void passf (long *nac,long ido,long ip,long l1,long idl1,
             double cc[],double c1[],double c2[],double ch[],
