@@ -1,13 +1,12 @@
 /*
- * CGM.C
- *
- * $Id: cgm.c,v 1.1 1993/08/27 17:08:43 munro Exp $
- *
+ * $Id: cgm.c,v 1.2 2006/08/04 04:40:23 dhmunro Exp $
  * Implement the CGM binary metafile engine for GIST.
- *
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
 #include "play.h"
 #include "pstdlib.h"
@@ -18,7 +17,7 @@
 #include <string.h>
 #include <time.h>
 
-static char *cgmType= "binary CGM";
+static g_callbacks g_cgm_on = { "gist CGMEngine", 0, 0, 0, 0, 0, 0, 0, 0 };
 
 #define N_CGMFONTS 20
 
@@ -31,7 +30,7 @@ extern double floor(double);
 /* ------------------------------------------------------------------------ */
 
 static Octet *FormCommand(Octet *buffer, int klass, int id, long nbytes,
-			  long *lpart);
+                          long *lpart);
 static Octet *NextPartition(Octet *buffer, long nbytes, long *lpart);
 static Octet *Pascalify(Octet *buffer, const char *text, long len, int pad);
 static Octet *FormWords(Octet *buffer, const short *words, long nwords);
@@ -43,7 +42,7 @@ static void WriteError(CGMEngine *cgm, const char *msg);
 static int BeginMetafile(CGMEngine *cgm);
 static void SetPageDefaults(CGMEngine *cgmEngine);
 static void SetCGMTransform(GpTransform *toPixels, int landscape,
-			    GpReal scale);
+                            GpReal scale);
 static int BeginPage(CGMEngine *cgmEngine);
 static void EndClip(CGMEngine *cgmEngine);
 static void BeginClip(CGMEngine *cgmEngine, GpTransform *trans);
@@ -59,19 +58,19 @@ static int SetupLine(CGMEngine *cgmEngine);
 static int SetupEdge(CGMEngine *cgmEngine);
 static void CheckClip(CGMEngine *cgmEngine);
 static int DrawLines(Engine *engine, long n, const GpReal *px,
-		     const GpReal *py, int closed, int smooth);
+                     const GpReal *py, int closed, int smooth);
 static int SetupMarker(CGMEngine *cgmEngine);
 static int DrawMarkers(Engine *engine, long n, const GpReal *px,
-		       const GpReal *py);
+                       const GpReal *py);
 static int SetupText(CGMEngine *cgmEngine);
 static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text);
 static int DrawFill(Engine *engine, long n, const GpReal *px,
-		    const GpReal *py);
+                    const GpReal *py);
 static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
-		     GpReal qy, long width, long height, long nColumns,
-		     const GpColor *colors);
+                     GpReal qy, long width, long height, long nColumns,
+                     const GpColor *colors);
 static int DrawDisjoint(Engine *engine, long n, const GpReal *px,
-			const GpReal *py, const GpReal *qx, const GpReal *qy);
+                        const GpReal *py, const GpReal *qx, const GpReal *qy);
 
 static void IncrementName(char *filename);
 
@@ -107,7 +106,7 @@ cgm_swap(short *words, long nwords)
 /* ------------------------------------------------------------------------ */
 
 static Octet *FormCommand(Octet *buffer, int klass, int id, long nbytes,
-			  long *lpart)
+                          long *lpart)
 {
   long nhere;
   *buffer++= (Octet)((klass<<4) | (id>>3));
@@ -246,7 +245,7 @@ static int BeginMetafile(CGMEngine *cgm)
   len+= 4;
   len+= 4 + lpart+1;
   if (len&1) len++;
-  len+= 4;
+  len+= 6*4;
   len+= 8;
   len+= 784;
   len+= 4 + lfonts;
@@ -398,7 +397,7 @@ static void SetPageDefaults(CGMEngine *cgmEngine)
 }
 
 static void SetCGMTransform(GpTransform *toPixels, int landscape,
-			    GpReal scale)
+                            GpReal scale)
 {
   toPixels->viewport.xmin= toPixels->viewport.ymin= 0.0;
   if (landscape) {
@@ -442,7 +441,7 @@ static int BeginPage(CGMEngine *cgmEngine)
   /* Set transform viewport to reflect current page orientation */
   if (cgmEngine->landscape != cgmEngine->e.landscape) {
     SetCGMTransform(&cgmEngine->e.transform, cgmEngine->e.landscape,
-		    cgmEngine->scale);
+                    cgmEngine->scale);
     cgmEngine->landscape= cgmEngine->e.landscape;
   }
 
@@ -705,9 +704,9 @@ static int SetupColor(CGMEngine *cgmEngine, unsigned long color, int which)
       if (color>=(unsigned long)nColors) color= nColors-1;
       if (cgmEngine->colorMode) c= (short)color;  /* page has color table */
       else {                              /* this is a 240 gray mono page */
-	c= (short)((P_R(palette[color]) +
+        c= (short)((P_R(palette[color]) +
                     P_G(palette[color]) + P_B(palette[color]))/3);
-	c-= (c+8)/16;
+        c-= (c+8)/16;
       }
     } else {
       if (color>255) color= 255;
@@ -848,7 +847,7 @@ static void CheckClip(CGMEngine *cgmEngine)
 }
 
 static int DrawLines(Engine *engine, long n, const GpReal *px,
-		     const GpReal *py, int closed, int smooth)
+                     const GpReal *py, int closed, int smooth)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -865,23 +864,23 @@ static int DrawLines(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nPoints=
-	  GpIntPoints(map, maxPoints, n, px, py, &points))) {
+          GpIntPoints(map, maxPoints, n, px, py, &points))) {
     if (closed) {
       if (firstPass) {
-	firstPoint= points[0];
-	firstPass= 0;
+        firstPoint= points[0];
+        firstPass= 0;
       }
       if (n==nPoints) {
-	n++;
-	points[nPoints++]= firstPoint;
+        n++;
+        points[nPoints++]= firstPoint;
       }
     }
 
     if (smooth) {
       /* There may be a GDP for Bezier curves, but I don't know how
-	 standard it is.  This is sure to be non-portable, but also
-	 sure to produces SOME acceptable output (identical, in fact,
-	 to what the Gist X engine displays).  */
+         standard it is.  This is sure to be non-portable, but also
+         sure to produces SOME acceptable output (identical, in fact,
+         to what the Gist X engine displays).  */
       short value= smooth;  /* identifier */
       now= FormCommand(command, 7, 2, 3L, &len);     /* APPLICATION DATA */
       now= FormWords(now, &value, 1L);
@@ -961,7 +960,7 @@ static int SetupMarker(CGMEngine *cgmEngine)
 }
 
 static int DrawMarkers(Engine *engine, long n, const GpReal *px,
-		       const GpReal *py)
+                       const GpReal *py)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -977,7 +976,7 @@ static int DrawMarkers(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nPoints=
-	  GpIntPoints(map, maxPoints, n, px, py, &points))) {
+          GpIntPoints(map, maxPoints, n, px, py, &points))) {
 
     now= FormCommand(command, 4, 3, 4*nPoints, &len);      /* POLYMARKER */
     if (WriteB(file, command, now-command)) {
@@ -1126,6 +1125,9 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
   long len;
   const char *t;
 
+  /* do not try to output zero size text */
+  if ((short)(0.8*gistA.t.height*cgmEngine->scale+0.5) <= 0) return 0;
+
   CheckClip(cgmEngine);
   if (SetupText(cgmEngine)) return 1;
   file= cgmEngine->file;
@@ -1152,10 +1154,10 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
   else { ymin= wind->ymax; ymax= wind->ymin; }
   if (gistA.t.orient==TX_RIGHT || gistA.t.orient==TX_LEFT) {
     if (x0<xmin-width || x0>xmax+width ||
-	y0<ymin-height || y0>ymax+height) return 0;
+        y0<ymin-height || y0>ymax+height) return 0;
   } else {
     if (x0<xmin-height || x0>xmax+height ||
-	y0<ymin-width || y0>ymax+width) return 0;
+        y0<ymin-width || y0>ymax+width) return 0;
   }
 
   /* Adjust y0 to represent topmost line */
@@ -1184,8 +1186,8 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
       now= FormWords(now, xyf, 3L);
       now= Pascalify(now, t, (long)count, 1);
       if (WriteB(file, cgmText, now-cgmText)) {
-	WriteError(cgmEngine, "write to CGM failed in DrwText");
-	return 1;
+        WriteError(cgmEngine, "write to CGM failed in DrwText");
+        return 1;
       }
     }
 
@@ -1202,7 +1204,7 @@ static int DrwText(Engine *engine, GpReal x0, GpReal y0, const char *text)
 /* ------------------------------------------------------------------------ */
 
 static int DrawFill(Engine *engine, long n, const GpReal *px,
-		    const GpReal *py)
+                    const GpReal *py)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -1222,7 +1224,7 @@ static int DrawFill(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nPoints=
-	  GpIntPoints(map, maxPoints, n, px, py, &points))) {
+          GpIntPoints(map, maxPoints, n, px, py, &points))) {
 
     now= FormCommand(command, 4, 7, 4*nPoints, &len);         /* POLYGON */
     if (WriteB(file, command, now-command)) {
@@ -1251,8 +1253,8 @@ static int DrawFill(Engine *engine, long n, const GpReal *px,
 /* ------------------------------------------------------------------------ */
 
 static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
-		     GpReal qy, long width, long height, long nColumns,
-		     const GpColor *colors)
+                     GpReal qy, long width, long height, long nColumns,
+                     const GpColor *colors)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -1271,10 +1273,10 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
   /* Transform corner coordinates, clipping and adjusting width,
      height, nColumns, and colors as necessary.  */
   width= GpClipCells(&map->x, &px, &qx,
-		     gistT.window.xmin, gistT.window.xmax, width, &off);
+                     gistT.window.xmin, gistT.window.xmax, width, &off);
   colors+= off;
   height= GpClipCells(&map->y, &py, &qy,
-		      gistT.window.ymin, gistT.window.ymax, height, &off);
+                      gistT.window.ymin, gistT.window.ymax, height, &off);
   colors+= nColumns*off;
   if (width>0x7ffe) width= 0x7ffe;
   if (height>0x7ffe) height= 0x7ffe;
@@ -1325,13 +1327,13 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
   for (;;) {
     while (lPart--) {
       if (i>=width) {
-	if (pad) {
-	  *now++= 0;
-	  nCells--;
-	  if (!lPart--) break;
-	}
-	i= 0;
-	j+= nColumns;
+        if (pad) {
+          *now++= 0;
+          nCells--;
+          if (!lPart--) break;
+        }
+        i= 0;
+        j+= nColumns;
       }
       if (gistA.rgb) {
         color = (colors[3*(i+j)]+colors[3*(i+j)+1]+colors[3*(i+j)+2])/3;
@@ -1342,9 +1344,9 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
       i++;
       if (color>=nColors && nColors>0) color= nColors-1;
       if (!colorMode) {  /* this is default 240 gray mono page */
-	color= (P_R(palette[color]) +
+        color= (P_R(palette[color]) +
                 P_G(palette[color]) + P_B(palette[color]))/3;
-	color-= (color+8)/16;
+        color-= (color+8)/16;
       }
 
       *now++= 10+color;  /* skip the 10 standard colors */
@@ -1361,6 +1363,7 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
 
     now= NextPartition(buffer, nCells, &lPart);
   }
+  p_free(buffer);
 
   return 0;
 }
@@ -1368,7 +1371,7 @@ static int DrawCells(Engine *engine, GpReal px, GpReal py, GpReal qx,
 /* ------------------------------------------------------------------------ */
 
 static int DrawDisjoint(Engine *engine, long n, const GpReal *px,
-			const GpReal *py, const GpReal *qx, const GpReal *qy)
+                        const GpReal *py, const GpReal *qx, const GpReal *qy)
 {
   CGMEngine *cgmEngine= (CGMEngine *)engine;
   p_file *file;
@@ -1384,7 +1387,7 @@ static int DrawDisjoint(Engine *engine, long n, const GpReal *px,
   file= cgmEngine->file;
 
   while ((nSegs=
-	  GpIntSegs(map, maxSegs, n, px, py, qx, qy, &segs))) {
+          GpIntSegs(map, maxSegs, n, px, py, qx, qy, &segs))) {
 
     now= FormCommand(command, 4, 2, 8*nSegs, &len); /* DISJOINT POLYLINE */
     if (WriteB(file, command, now-command)) {
@@ -1417,7 +1420,7 @@ static void IncrementName(char *filename)
 {
   int i, len= filename? strlen(filename) : 0;
   if (len>4 && (strcmp(filename+len-4, ".cgm")==0 ||
-		strcmp(filename+len-4, ".CGM")==0)) i= len-4;
+                strcmp(filename+len-4, ".CGM")==0)) i= len-4;
   else i= len;
   while (i-- > 0) {
     if (filename[i]=='9') {
@@ -1433,7 +1436,7 @@ static void IncrementName(char *filename)
 /* ------------------------------------------------------------------------ */
 
 GpReal gCGMScale= 25545.24;   /* default CGM scale is 2400 dpi (round up) */
-long gCGMFileSize= 1000000;   /* default max file size is about 1 Meg */
+long gCGMFileSize= 100000000; /* default max file size is about 100 Meg */
 
 Engine *GpCGMEngine(char *name, int landscape, int mode, char *file)
 {
@@ -1447,11 +1450,11 @@ Engine *GpCGMEngine(char *name, int landscape, int mode, char *file)
   SetCGMTransform(&toPixels, landscape, gCGMScale);
 
   cgmEngine=
-    (CGMEngine *)GpNewEngine(engineSize, name, cgmType, &toPixels, landscape,
-			     &Kill, &Clear, &Flush, &GpComposeMap,
-			     &ChangePalette, &DrawLines, &DrawMarkers,
-			     &DrwText, &DrawFill, &DrawCells,
-			     &DrawDisjoint);
+    (CGMEngine *)GpNewEngine(engineSize, name, &g_cgm_on, &toPixels, landscape,
+                             &Kill, &Clear, &Flush, &GpComposeMap,
+                             &ChangePalette, &DrawLines, &DrawMarkers,
+                             &DrwText, &DrawFill, &DrawCells,
+                             &DrawDisjoint);
 
   if (!cgmEngine) {
     strcpy(gistError, "memory manager failed in GpCGMEngine");
@@ -1479,7 +1482,7 @@ Engine *GpCGMEngine(char *name, int landscape, int mode, char *file)
 
 CGMEngine *GisCGMEngine(Engine *engine)
 {
-  return (engine && engine->type==cgmType)? (CGMEngine *)engine : 0;
+  return (engine && engine->on==&g_cgm_on)? (CGMEngine *)engine : 0;
 }
 
 void GcgmSetScale(CGMEngine *cgmEngine, GpReal scale)
