@@ -1,9 +1,13 @@
 /*
-    CLIP.C
-    Routines to perform floating point clipping operations.
-
-    $Id: clip.c,v 1.1 1993/08/27 17:08:43 munro Exp $
-
+ * $Id: clip.c,v 1.1.1.1 2005/09/18 22:04:25 dhmunro Exp $
+ * Routines to perform floating point clipping operations.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
+/*
     Interface routines:
 
     *** include "clip.h" to define xClip, yClip ***
@@ -28,8 +32,6 @@
     if (nc=ClipDisjoint(x0, y0, x1, y1, n))
        DrawDisjoint(xClip, yClip, xClip1, yClip1, nc);
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "clip.h"
 
