@@ -1,11 +1,12 @@
 /*
-   DRAW0.C
-   Virtual functions for Drauing class.
-
-   $Id$
+ * $Id: draw0.c,v 1.1.1.1 2005/09/18 22:04:29 dhmunro Exp $
+ * Virtual functions for Drauing class.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "draw.h"
 #include "gtext.h"
@@ -1053,10 +1054,11 @@ static int DisjointScan(void *el, int flags, GpBox *limits)
     e->el.box.xmin= e->logBox.xmin;
     e->el.box.xmax= e->logBox.xmax;
     x= e->xlog;
-    if (!e->xqlog && GetLogZ(e->n, e->xq, &e->xqlog,
-                             &xymin, &xymax)) return 1;
-    if (xymin<e->el.box.xmin) e->el.box.xmin= e->logBox.xmin;
-    if (xymax>e->el.box.xmax) e->el.box.xmax= e->logBox.xmax;
+    if (!e->xqlog) {
+      if (GetLogZ(e->n, e->xq, &e->xqlog, &xymin, &xymax)) return 1;
+      if (xymin<e->el.box.xmin) e->el.box.xmin= e->logBox.xmin;
+      if (xymax>e->el.box.xmax) e->el.box.xmax= e->logBox.xmax;
+    }
     xq= e->xqlog;
   } else {
     e->el.box.xmin= e->linBox.xmin;
@@ -1070,10 +1072,11 @@ static int DisjointScan(void *el, int flags, GpBox *limits)
     e->el.box.ymin= e->logBox.ymin;
     e->el.box.ymax= e->logBox.ymax;
     y= e->ylog;
-    if (!e->yqlog && GetLogZ(e->n, e->yq, &e->yqlog,
-                             &xymin, &xymax)) return 1;
-    if (xymin<e->el.box.ymin) e->el.box.ymin= e->logBox.ymin;
-    if (xymax>e->el.box.ymax) e->el.box.ymax= e->logBox.ymax;
+    if (!e->yqlog) {
+      if (GetLogZ(e->n, e->yq, &e->yqlog, &xymin, &xymax)) return 1;
+      if (xymin<e->el.box.ymin) e->el.box.ymin= e->logBox.ymin;
+      if (xymax>e->el.box.ymax) e->el.box.ymax= e->logBox.ymax;
+    }
     yq= e->yqlog;
   } else {
     e->el.box.ymin= e->linBox.ymin;
