@@ -1,13 +1,12 @@
 /*
- * XBASIC.C
- *
- * $Id: xbasic.c,v 1.1 1993/08/27 17:08:43 munro Exp $
- *
+ * $Id: xbasic.c,v 1.3 2007/12/28 20:20:18 thiebaut Exp $
  * Implement the basic X windows engine for GIST.
- *
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
 #include "xbasic.h"
 #include "gtext.h"
@@ -73,6 +72,11 @@ extern int GxJustifyNext(const char **text, int *ix, int *iy);
 static int gxErrorFlag= 0;
 static void GxErrorHandler(void);
 
+/* Engine which currently has mouse focus (set to NULL when the
+   "current" engine get destroyed or when mouse leaves the "current"
+   engine window, set to engine address on mouse motion). */
+Engine *gxCurrentEngine = NULL;
+
 /* ------------------------------------------------------------------------ */
 
 static int
@@ -443,6 +447,8 @@ GxJustifyNext(const char **text, int *ix, int *iy)
  *       -- thus, the p_disconnect must take place on the next
  *          event, which is during the GhBeforeWait hlevel.c function
  */
+/* hack to disconnect if last engine destroyed (see GhBeforeWait) */
+static void g_do_disconnect(void);
 extern void (*g_pending_task)(void);
 void (*g_pending_task)(void) = 0;
 
@@ -452,12 +458,9 @@ Kill(Engine *engine)
   XEngine *xeng= (XEngine *)engine;
   p_win *w = xeng->win;
   ShutDown(xeng);
-  if (w) {
-    xeng->w = 0;
-    p_destroy(w);
-  }
+  if (w) p_destroy(w);
   /* for program-driven Kill(), can take care of p_disconnect immediately */
-  if (g_pending_task) g_pending_task();
+  g_do_disconnect();
 }
 
 static int
@@ -1093,6 +1096,7 @@ static void
 g_on_motion(void *c,int md,int x,int y)
 {
   XEngine *xeng = c;
+  gxCurrentEngine = (Engine *)c;
   if (xeng->e.on==&g_x_on) {
     if (!xeng->w) return;
     if (xeng->HandleMotion)
@@ -1129,6 +1133,7 @@ static void
 g_on_focus(void *c,int in)
 {
   XEngine *xeng = c;
+  if (in == 2) gxCurrentEngine = NULL; /* current window has lost mouse focus */
   if (xeng->e.on==&g_x_on) {
     if (xeng->w && xeng->HandleMotion && in==2)
       xeng->HandleMotion(&xeng->e, 0, -1, -1);
@@ -1304,9 +1309,6 @@ g_connect(char *displayName)
   return s;
 }
 
-/* hack to disconnect if last engine destroyed (see GhBeforeWait) */
-static void g_do_disconnect(void);
-
 void
 g_disconnect(p_scr *s)
 {
@@ -1405,6 +1407,9 @@ int gx100width = 600;
 int gx75height = 450;
 int gx100height = 600;
 
+unsigned long gx_parent = 0;
+int gx_xloc=0, gx_yloc=0;
+
 int gist_private_map = 0;
 int gist_input_hint = 0;
 int gist_rgb_hint = 0;
@@ -1436,8 +1441,11 @@ GpBXEngine(char *name, int landscape, int dpi, char *displayName)
   /* possibly want optional P_RGBMODEL as well */
   hints = (gist_private_map?P_PRIVMAP:0) | (gist_input_hint?0:P_NOKEY) |
     (gist_rgb_hint?P_RGBMODEL:0);
-  xeng->win = xeng->w =
+  xeng->win = xeng->w = gx_parent?
+    p_subwindow(s, topWidth, topHeight,
+                gx_parent, gx_xloc, gx_yloc, P_BG, hints, xeng) :
     p_window(s, topWidth, topHeight, name, P_BG, hints, xeng);
+  gx_parent = 0;
   if (!xeng->win) {
     GpDelEngine(&xeng->e);
     return 0;
@@ -1474,7 +1482,7 @@ int
 GxAnimate(Engine *engine, GpBox *viewport)
 {
   XEngine *xeng = GisXEngine(engine);
-  int x, y, x0, y0, x1, y1;
+  int x, y, x1, y1;
   GpBox *v, *w;
   GpReal xmin, xmax, ymin, ymax;
   GpReal scalx, offx, scaly, offy;
@@ -1532,9 +1540,12 @@ GxAnimate(Engine *engine, GpBox *viewport)
     w->ymax = 0.0;
   }
   GpDeviceMap((Engine *)xeng);
-  GetXRectangle(&xeng->e.devMap, v, &x0, &y0, &x1, &y1);
-  x1 -= x0;
-  y1 -= y0;
+  /* GetXRectangle(&xeng->e.devMap, v, &x0, &y0, &x1, &y1);
+     x1 -= x0;
+     y1 -= y0;
+  */
+  x1 = xeng->wtop;
+  y1 = xeng->htop;
 
   /* create the offscreen pixmap */
   xeng->w = p_offscreen(xeng->win, x1, y1);
@@ -1642,6 +1653,7 @@ ShutDown(XEngine *xeng)
   p_scr *s = xeng->s;
   p_win *w = xeng->w;
   p_win *win = xeng->win;
+  if ((Engine *)xeng == gxCurrentEngine) gxCurrentEngine = NULL;
   xeng->mapped= 0;
   /* turn off all event callbacks (probably unnecessary) */
   xeng->e.on = 0;
@@ -1701,12 +1713,26 @@ g_rgb_read(Engine *eng, GpColor *rgb, long *nx, long *ny)
 {
   XEngine *xeng = GisXEngine(eng);
   if (!xeng || !xeng->w || !xeng->win) return 1;
-  if (!rgb) {
-    *nx = xeng->wtop;
-    *ny = xeng->htop;
+  GpPreempt(eng);
+  GdDraw(1);       /* make sure screen updated */
+  GpPreempt(0);
+  if (xeng->w == xeng->win) {
+    /* not in animate mode */
+    if (!rgb) {
+      *nx = xeng->wtop;
+      *ny = xeng->htop;
+    } else {
+      p_rgb_read(xeng->win, rgb, xeng->leftMargin, xeng->topMargin,
+                 xeng->leftMargin+xeng->wtop, xeng->topMargin+xeng->htop);
+    }
   } else {
-    p_rgb_read(xeng->win, rgb, xeng->leftMargin, xeng->topMargin,
-               xeng->leftMargin+xeng->wtop, xeng->topMargin+xeng->htop);
+    /* in animate mode, read offscreen pixmap */
+    if (!rgb) {
+      *nx = xeng->a_width;
+      *ny = xeng->a_height;
+    } else {
+      p_rgb_read(xeng->w, rgb, 0, 0, xeng->a_width, xeng->a_height);
+    }
   }
   return 0;
 }
