@@ -1,8 +1,11 @@
 /*
-   DGESVD.C
-   LAPACK routine to return the SVD of a matrix (m>=n branch only).
-
-   $Id$
+ * $Id: dgesvd.c,v 1.1.1.1 2005/09/18 22:04:48 dhmunro Exp $
+ * LAPACK routine to return the SVD of a matrix (m>=n branch only).
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
 
 #include "dg.h"
@@ -16,44 +19,6 @@ extern void dgesv2( char jobu, char jobvt, long m, long n,
                    double work[], long lwork, long *info );
 
 
-/*-----prototypes of functions defined here-----*/
-extern void dgesvd( char jobu, char jobvt, long m, long n,
-                   double a[], long lda, double s[], double u[], long ldu,
-                   double vt[], long ldvt,
-                   double work[], long lwork, long *info );
-/*-----end of prototypes-----*/
-
-
-/*---other lapack routines---*/
-extern double   dlamch( char cmach );
-extern long ilaenv( long ispec, char *name, char *opts,
-                   long n1, long n2, long n3,long n4 );
-extern double   dlange( char norm, long m, long n, double a[], long lda,
-                       double work[] );
-extern void dgeqrf( long m, long n, double a[], long lda, double tau[],
-                   double work[], long lwork, long *info );
-extern void dlascl( char type, long kl, long ku, double cfrom, double cto,
-                   long m, long n, double a[], long lda, long *info );
-extern void dlaset( char uplo, long m, long n, double alpha, double beta,
-                   double a[], long lda );
-extern void dlacpy( char uplo, long m, long n, double a[], long lda,
-                   double b[], long ldb );
-extern void dbdsqr( char uplo, long n, long ncvt, long nru, long ncc,
-                   double d[], double e[], double vt[], long ldvt,
-                   double u[],long ldu, double c[], long ldc,
-                   double work[], long *info );
-extern void dgebrd( long m, long n, double a[], long lda,
-                   double d[], double e[], double tauq[], double taup[],
-                   double work[], long lwork,long *info );
-extern void dorgbr( char vect, long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long lwork, long *info );
-extern void dorgqr( long m, long n, long k, double a[], long lda,
-                   double tau[], double work[], long lwork, long *info );
-extern void dormbr( char vect, char side, char trans, long m, long n, long k,
-                   double a[], long lda, double tau[], double c[],long ldc,
-                   double work[], long lwork, long *info );
-
-
 
 /*---blas routines---*/
 /* dgemm */
@@ -63,9 +28,6 @@ extern void dormbr( char vect, char side, char trans, long m, long n, long k,
 /*---converted nutty string switches to single characters (lower case)---*/
 #define lsame(x,y) ((x)==(y))
 
-extern void xerbla(char *,long);
-
-
 
 /*-----Fortran intrinsics converted-----*/
 extern double sqrt(double);
