@@ -1,13 +1,14 @@
 /*
-    ASCIO.C
-    Define standard Yorick built-in functions for ASCII I/O
-
-    See std.i for documentation on the interface functions defined here.
-
-    $Id: ascio.c,v 1.1 1993/08/27 18:32:09 munro Exp munro $
+ * $Id: ascio.c,v 1.3 2007/02/24 23:14:41 dhmunro Exp $
+ * Define standard Yorick built-in functions for ASCII I/O
+ *
+ * See std.i for documentation on the interface functions defined here.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 #include "yio.h"
@@ -97,15 +98,6 @@ static PrtFunc *PrintD[]= { &FPrintD, &DPrintD };
 /* Two data types which are "foreign" to Yorick are defined in this
    file: the TextStream and the Bookmark.  */
 
-extern PromoteOp PromXX;
-extern UnaryOp ToAnyX, NegateX, ComplementX, NotX, TrueX;
-extern BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
-extern BinaryOp EqualX, NotEqualX, GreaterX, GreaterEQX;
-extern BinaryOp ShiftLX, ShiftRX, OrX, AndX, XorX;
-extern BinaryOp AssignX, MatMultX;
-extern UnaryOp EvalX, SetupX, PrintX;
-extern MemberOp GetMemberX;
-
 static UnaryOp PrintBM, PrintTX;
 
 /* Implement text streams as a foreign Yorick data type.  */
@@ -219,10 +211,10 @@ void Y_open(int nArgs)
 
   } else {
     /* blow up if optional errmode flag is not set */
-    char *dots= strlen(filename)>40? "..." : "";
-    char message[80];
+    char *dots= strlen(filename)>100? "..." : "";
+    char message[140];
     p_free(fullname);
-    sprintf(message, "cannot open file %.40s%s (mode %.6s)",
+    sprintf(message, "cannot open file %.100s%s (mode %.6s)",
             filename, dots, filemode);
     YErrorIO(message);
     return;
@@ -289,7 +281,6 @@ void Y_close(int nArgs)
  */
 
 extern void yr_reader(char *input_line);
-extern void yr_reset(void);
 extern Instruction *ym_suspend(void);
 extern void ym_resume(Instruction *pc);
 extern char *y_read_prompt;
@@ -454,7 +445,7 @@ ReadWorker(Operand *sourceOp, Symbol *stack, char *format, Symbol *stack0)
 
   /* First pass through arguments counts them, checks data types and
      conformability, and matches them to conversions in the format list.  */
-  CheckOps((int)(sp-stack+1));
+  CheckOps((int)(sp-stack+2));
   nArgs= 0;
   for ( ; stack<=sp ; stack++) {
     if (!stack->ops) { /* skip keywords */
@@ -513,7 +504,13 @@ ReadWorker(Operand *sourceOp, Symbol *stack, char *format, Symbol *stack0)
                  fmtList[nArgs].format[i-1]=='\015' ||
                  fmtList[nArgs].format[i-1]==' '))
       fmtList[nArgs].format[--i]= '\0';  /* strip off trailing newlines */
-    if (i) nArgs++;
+    if (i) {
+      if (from_keybd) {
+        ioOps[nArgs].s.ops = 0;
+        ioOps[nArgs].s.index = -1;
+      }
+      nArgs++;
+    }
   }
 
   if (from_keybd) {
@@ -574,7 +571,7 @@ void Y_rdline(int nArgs)
   long i, nLines= 0;
   Array *result;
   Dimension *dims;
-  char *keyNames[2];
+  char *keyNames[2], *q;
   Symbol *keySymbols[1];
   Symbol *stack0= sp-nArgs;
 
@@ -638,8 +635,11 @@ void Y_rdline(int nArgs)
     return;
   }
 
-  for (i=0 ; i<nLines ; i++)
-    result->value.q[i]= p_strcpy(NextInLine(&op, i));
+  for (i=0 ; i<nLines ; i++) {
+    q = NextInLine(&op, i);
+    if (!q) break;
+    result->value.q[i] = p_strcpy(q);
+  }
 }
 
 /*--------------------------------------------------------------------------*/
@@ -1039,10 +1039,11 @@ static char *NextInLine(Operand *op, long n)
         YErrorIO("****ABORTING READ**** error reading input file");
       if (!hadEOF)
         YErrorIO("****ABORTING READ**** input file not ASCII text");
+    } else {
+      stream->lastLineRead++;
+      stream->lastPosition= stream->readPosition;
+      if (!(stream->permissions&16)) stream->readPosition= p_ftell(file);
     }
-    stream->lastLineRead++;
-    stream->lastPosition= stream->readPosition;
-    if (!(stream->permissions&16)) stream->readPosition= p_ftell(file);
   } else if (op->ops==&stringOps) {
     char **q= op->value;
     if (n<op->type.number) text= q[n];
