@@ -1,11 +1,12 @@
 /*
-    BCAST.C
-    Implement generic broadcast, scatter, and gather functions.
-
-    $Id: bcast.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: bcast.c,v 1.1.1.1 2005/09/18 22:03:48 dhmunro Exp $
+ * Implement generic broadcast, scatter, and gather functions.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "bcast.h"
 
@@ -121,8 +122,9 @@ static int BlockCast(const Dimension *ddims, const Dimension *sdims,
     /* must recurse to do a sparse copy */
     char *saddr0= saddr;
     long i;
-    while (nCopies--) {
+    for (;;) {
       for (i=0 ; i<n ; i++) BlockCast(ddims, sdims, 1L);
+      if (! --nCopies) break;
       saddr= saddr0;
     }
 
