@@ -1,11 +1,12 @@
 /*
-    BINOBJ.C
-    Implement functions for generic blocks of data.
-
-    $Id: binobj.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: binobj.c,v 1.1.1.1 2005/09/18 22:03:50 dhmunro Exp $
+ * Implement functions for generic blocks of data.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "bcast.h"
 #include "defmem.h"
@@ -152,6 +153,15 @@ Dimension *NewDimension(long number, long origin, Dimension *next)
   return dims;
 }
 
+Dimension *
+ynew_dim(long n, Dimension *next)
+{
+  Dimension *dims = tmpDims;
+  tmpDims = 0;
+  if (dims) FreeDimension(dims);
+  return tmpDims = NewDimension(n, 1L, next);
+}
+
 void FreeDimension(Dimension *dims)
 {
   if (dims && --dims->references<0) {
