@@ -1,11 +1,12 @@
 /*
-    BINPDB.C
-    Define Yorick interface to PDB files
-
-    $Id: binpdb.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: binpdb.c,v 1.2 2006/05/12 03:35:48 dhmunro Exp $
+ * Define Yorick interface to PDB files
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #ifdef NOT_YORICK
 #include "binio.h"
@@ -54,16 +55,10 @@ static int yPDBclose;  /* options for closing a PDB file, selected by
 /*--------------------------------------------------------------------------*/
 /*--------------------------------------------------------------------------*/
 
-/*
- * Based on code bearing the following copyright notices:
- *
- * Copyright Stewart A. Brown 1987-1990 (all rights reserved)
- *
- * The U.S. Government, for itself and others acting on its behalf, has
- * been granted a royalty-free, nonexclusive, irrevocable, world-wide
- * license for Governmental purposes to publish, distribute, translate,
- * duplicate, exhibit, and perform this material.
- *
+/* The PDB file format was invented by Stewart Brown of Lawrence
+ * Livermore Laboratory.  This interface to the PDB format is entirely
+ * my own (David Munro); Stewart Brown's PDBlib API is a part of his PACT
+ * library, which you can obtain from http://pact.llnl.gov.
  */
 
 /*--------------------------------------------------------------------------*/
@@ -137,7 +132,7 @@ static void Put1Dim(Dimension *dims, int firstPass);
 static int DoStructName(char *name, long size, int putEnd);
 static int WRsymtab(IOStream *file);
 static int WRsymbol(char *name, char *typeName, long number, long address,
-                    Dimension *dims);
+                    Dimension *dims, long nlen);
 static void Put102Dims(Dimension *dims);
 static int WRextras(IOStream *file);
 static int WRheader(IOStream *file, long newChart, long newSymtab);
@@ -153,6 +148,8 @@ static char *variablePointer= "char*";
 static long mptrSize;
 static int mptrAlign;
 
+static int has_dirs;
+
 /* state information for DMI-style record structure in PDB file */
 static long firstLength, nSeq, nSeq2, nTimes, nCycles;
 static long firstAddr, numbAddr, timeAddr, ncycAddr;
@@ -302,6 +299,7 @@ int YtestPDB(IOStream *file, int familyOK)
      present, RDsymtab uses this blocksTable to identify history data.  */
   BlocksInit();
   yPDBclose= 06;   /* initialize file style here */
+  has_dirs = 0;
   RDextras(file, symtabAddress);
   if (yPDBopen&3) {
     if ((yPDBopen&3)==1) fastestFirst= 1;
@@ -332,6 +330,7 @@ int YtestPDB(IOStream *file, int familyOK)
      set file->nextAddress intelligently...  */
   DMIhistory(file, chartAddress, symtabAddress, familyOK);
 
+  has_dirs = 0;
   BlocksInit();
   return ClearBuffer(0);
 }
@@ -397,6 +396,28 @@ void YPDBpointers(IOStream *file, long size, int align)
                     &pointerStruct, &PDBconvert);
     InstallStruct(base, (StructDef *)0);
   }
+  /* note: before yorick-1.6, always wrote just member and variable pointer
+   * in addition to the 8 automatically defined types, for a total of 10
+   * types always assumed present
+   * for a short time until mid-yorick-1.6.03, created Directory here
+   * as an 11th type, which causes no problems until you attempt to read
+   * a file written with yorick-1.5 containing pointers to objects with
+   * structs beyond the original total of 10 (either complex or a true
+   * struct) - at that ponit the numbering of the structs in the 1.6
+   * yorick did not match the numbering in 1.5, preventing 1.6 from
+   * correctly interpreting such 1.5 files
+   * -the reverse operation, reading 1.6.02 files with 1.5 works correctly,
+   *  because the Directory struct is written into the file as
+   *  the eleventh type, which it really is
+   * hence, for backward compatibility, must always define precisely
+   * the original 10 data type automatically, no more and no less,
+   * defining "Directory" if and only if it is actually used
+   * - interestingly, none of the files is actually incorrect, the
+   *   problem is that yorick pointee headers assume an offset of
+   *   exactly 10 to the first user-defined struct/primitive
+   *   - this is poor design, but hard to fix given that the PrimitiveType
+   *     and struct order is defined only by the yorick source code
+   */
 }
 
 /*--------------------------------------------------------------------------*/
@@ -437,6 +458,7 @@ static void ClosePDB(IOStream *file)
   if (!(file->permissions&2) ||
       (file->history &&
        file->history->fileNumber!=file->history->nFamily-1)) return;
+  has_dirs = 0;
 
   FreeClogFile(file);  /* don't want to log this stuff... */
 
@@ -458,6 +480,7 @@ static void ClosePDB(IOStream *file)
 
   notOK|= WRheader(file, chartAddress, symtabAddress);
 
+  has_dirs = 0;
   if (notOK) {
     /* don't try to call YclosePDB again */
     if (!file->history) file->CloseHook= 0;
@@ -776,6 +799,10 @@ static void RDextras(IOStream *file, long address)
       long code;
       if (TokenAsLong(&line[12], &code)==0 && code==102) fastestFirst= 1;
 
+    } else if (strncmp(line, "Has-Directories:", 16L)==0) {
+      long hd;
+      if (TokenAsLong(&line[16], &hd)==0 && hd!=0) has_dirs = 1;
+
     } else if (strncmp(line, "Primitive-Types:", 16L)==0) {
       /* This one reads several more ANY_NEWLINE delimited lines...  */
       char *typeName;
@@ -2179,7 +2206,7 @@ static int WRsymtab(IOStream *file)
   HistoryInfo *history= file->history;
   IOStream *parent= history? history->parent : file;
   StructDef *base;
-  long i;
+  long i, dir_index;
 
   int notOK= 0;
 
@@ -2192,12 +2219,18 @@ static int WRsymtab(IOStream *file)
     long offset= parent->offset;
     char *typeName, **tNames= parent->structTable.names;
 
+    if (HashFind(&parent->structTable, "Directory", 0L))
+      dir_index = hashIndex;
+    else
+      dir_index = -1;
+
     for (i=0 ; i<nItems ; i++) {
       base= members[i].base;
       if (base->Convert!=&PDBconvert) typeName= tNames[base->index];
       else typeName= variablePointer;
+      if (!has_dirs && base->index==dir_index) has_dirs = 1;
       notOK|= WRsymbol(names[i], typeName, members[i].number,
-                       addresses[i]+offset, members[i].dims);
+                       addresses[i]+offset, members[i].dims, 0L);
     }
   }
 
@@ -2215,7 +2248,7 @@ static int WRsymtab(IOStream *file)
       for (i=0 ; i<dmiNumber ; i++) {
         sprintf(recordName, "__%05ld", dmiFirst+i);
         notOK|= WRsymbol(recordName, "__", 1L,
-                         offset[dmiFirst+i], (Dimension *)0);
+                         offset[dmiFirst+i], (Dimension *)0, 0L);
       }
 
       /* firstAddr, numbAddr, timeAddr, and ncycAddr have already been
@@ -2227,18 +2260,18 @@ static int WRsymtab(IOStream *file)
       tmpDims= NewDimension(1L, 1L, tmpDims);
       if (firstAddr)
         notOK|= WRsymbol("FirstRecordNames", "char", firstLength,
-                         firstAddr, tmpDims);
+                         firstAddr, tmpDims, 0L);
       dims= tmpDims->next;
       tmpDims->next= 0;
       FreeDimension(dims);
       if (numbAddr)
-        notOK|= WRsymbol("NumberOfRecords", "long", 1L, numbAddr, tmpDims);
+        notOK|= WRsymbol("NumberOfRecords","long", 1L, numbAddr, tmpDims, 0L);
       if (dmiNumber) {
         tmpDims->number= dmiNumber;
         if (timeAddr)
-          notOK|= WRsymbol("times", "double", dmiNumber, timeAddr, tmpDims);
+          notOK|= WRsymbol("times","double",dmiNumber, timeAddr, tmpDims, 0L);
         if (ncycAddr)
-          notOK|= WRsymbol("ncycs", "long", dmiNumber, ncycAddr, tmpDims);
+          notOK|= WRsymbol("ncycs", "long", dmiNumber, ncycAddr, tmpDims, 0L);
       }
 
     } else {
@@ -2256,6 +2289,11 @@ static int WRsymtab(IOStream *file)
         long *addresses= child->addresses;
         char *typeName, **tNames= child->structTable.names;
 
+        if (HashFind(&child->structTable, "Directory", 0L))
+          dir_index = hashIndex;
+        else
+          dir_index = -1;
+
         if (recordVar) {
           name= recordVar;
           recordVar= 0;
@@ -2269,8 +2307,9 @@ static int WRsymtab(IOStream *file)
           if (yPDBclose&4) name= recordVar= p_strncat(names[i], "@history", 0);
           else name= names[i];
           tmpDims= NewDimension(1L, 1L, Ref(members[i].dims));
+          if (!has_dirs && base->index==dir_index) has_dirs = 1;
           notOK|= WRsymbol(name, typeName, members[i].number,
-                           addresses[i]+offset[dmiFirst], tmpDims);
+                           addresses[i]+offset[dmiFirst], tmpDims, 0L);
           dims= tmpDims;
           tmpDims= 0;
           FreeDimension(dims);
@@ -2285,7 +2324,7 @@ static int WRsymtab(IOStream *file)
            -- name is __@history whether or not yPDBclose&4 is set */
         tmpDims= NewDimension(1L, 1L, tmpDims);
         notOK|= WRsymbol("__@history", "__", 1,
-                         offset[dmiFirst], (Dimension *)0);
+                         offset[dmiFirst], (Dimension *)0, 0L);
       }
     }
   }
@@ -2296,16 +2335,20 @@ static int WRsymtab(IOStream *file)
 }
 
 static int WRsymbol(char *name, char *typeName, long number, long address,
-                    Dimension *dims)
+                    Dimension *dims, long nlen)
 {
   int notOK= 0;
+  if (!nlen) nlen = strlen(name);
+  typeName = FlopIntegerAndInt(typeName);
 
-  if (CheckBuffer(45+strlen(name)+strlen(typeName))) {
+  if (CheckBuffer(45+nlen+strlen(typeName))) {
     YWarning("impossibly long type+variable name skipped closing PDB file");
     notOK= 1;
   } else {
-    sprintf(nextByte, "%s\001%s\001%ld\001%ld\001",
-            name, FlopIntegerAndInt(typeName), number, address);
+    nextByte[0] = '\0';
+    strncat(nextByte, name, nlen);
+    nextByte += nlen;
+    sprintf(nextByte, "\001%s\001%ld\001%ld\001", typeName, number, address);
     nextByte+= strlen(nextByte);
     if (!(yPDBclose&1)) {
       for ( ; dims ; dims=dims->next) {
@@ -2385,7 +2428,8 @@ static int WRextras(IOStream *file)
   /* Major-Order must come before Blocks for PDBLib */
   sprintf(nextByte, "Major-Order:%s\012", (yPDBclose&1)? "102" : "101");
   nextByte+= 16;
-  sprintf(nextByte, "Has-Directories:0\012");
+  sprintf(nextByte, "Has-Directories:%d\012",
+          HashFind(&file->dataTable, "/", 1L)?1:0);
   nextByte+= 18;
 
   /* blocks information optional as history mechanism */
