@@ -1,12 +1,13 @@
 /*
-    CACHE.C
-    Define caching (disk buffering) scheme used for random access
-    binary I/O.
-
-    $Id: cache.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: cache.c,v 1.4 2007/08/04 01:03:01 dhmunro Exp $
+ * Define caching (disk buffering) scheme used for random access
+ * binary I/O.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "binio.h"
 #include "pstdlib.h"
@@ -234,6 +235,7 @@ static long RawWrite(IOStream *file, const void *buf, long addr, long len)
         YcWrite(file, bufc, addr, first); /* clobbers wrtBuffer! */
         addr+= first;
         bufc+= first;
+        block = file->blockList;   /* YcWrite may clobber blockList! */
       }
       /* Must release any cache blocks in this region before this write
          (or update and mark them as not dirty).  */
@@ -241,7 +243,9 @@ static long RawWrite(IOStream *file, const void *buf, long addr, long len)
       while (block && block->nextAddress>addr) {
         prevBlock= block->prev;
         FreeCacheBlock(block);  /* take easy way out for now... */
+        block = prevBlock;
       }
+      prevBlock = 0;
       file->ioOps->Seek(file, addr);
       file->ioOps->Write(file, bufc, sizeof(char), final-addr);
       if (last>final) {
@@ -513,6 +517,9 @@ static void FreeCacheBlock(CacheBlock *block)
   else if (file) file->blockList= block->prev;
   if (block->prev) block->prev->next= block->next;
 
+  /* probably impossible, but best to be safe */
+  if (block == prevBlock) prevBlock = 0;
+
   p_free(block);
 }
 
