@@ -1,11 +1,12 @@
 /*
-    CONVRT.C
-    Implement primitive data type conversion.
-
-    $Id: convrt.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: convrt.c,v 1.1.1.1 2005/09/18 22:04:00 dhmunro Exp $
+ * Implement primitive data type conversion.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "binio.h"
 #include "pstdlib.h"
