@@ -1,12 +1,13 @@
 /*
-    DEBUG.C
-
-    Define Yorick debugging functions.
-
-    $Id: debug.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: debug.c,v 1.3 2007/04/06 22:04:33 thiebaut Exp $
+ *
+ * Define Yorick debugging functions.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 #include "yio.h"
@@ -44,7 +45,7 @@ extern RangeFunc RFmin, RFmax, RFptp, RFsum, RFavg, RFrms, RFmnx, RFmxx,
 extern char *GetRFName(RangeFunc *rfTarget);  /* from yio.c */
 extern int PutsAsArray(char *s);              /* from yio.c */
 
-extern UnaryOp PrintXFN;
+extern UnaryOp PrintFN;
 
 /* virtual machine instructions */
 extern VMaction PushChar, PushShort, PushInt, PushLong,
@@ -370,7 +371,7 @@ void Y_disassemble(int nArgs)
   VMaction *Action;
   int stack, iAction, category;
   long ipc;
-  Operand op;   /* PrintXFN uses only op.value */
+  Operand op;   /* PrintFN uses only op.value */
 
   if (nArgs > 1) YError("disassemble takes a single argument");
   if (nArgs==1 && sp->ops==&referenceSym) ReplaceRef(sp);
@@ -395,7 +396,7 @@ void Y_disassemble(int nArgs)
     PushDataBlock(NewArray(&stringStruct, (Dimension *)0));
   }
 
-  PrintXFN(&op);
+  PrintFN(&op);
   ForceNewline();
 
   /* skip past local variables */
@@ -528,7 +529,7 @@ int YpReCompare(Function *func,
 
   match= (func->nPos==nPos && func->nKey==nKey && func->nLocal==nLocal &&
           func->hasPosList==hasPL && func->nConstants==nConsts &&
-          func->nReq==frameSize+maxStackDepth+2);
+          func->nReq==frameSize+maxStackDepth+10);
 
   if (match) {
     codeSize-= frameSize-1;
@@ -593,7 +594,6 @@ struct DebugBlk {
 extern BuiltIn Y_dbexit, Y_dbcont, Y_dbret, Y_dbskip, Y_dbup;
 extern BuiltIn Y_dbinfo, Y_dbdis, Y_dbauto;
 
-extern void ResetStack(int hard);
 extern Instruction *AbortReturn(void);
 extern int yAutoDebug;
 extern char *MakeErrorLine(long lineNumber, const char *filename);
@@ -606,15 +606,6 @@ static long FindErrLine(DebugBlk *dbg);
 static DebugBlk *FindDebugBlk(void);
 static void AnnouncePC(DebugBlk *dbg, int dontPush);
 
-extern PromoteOp PromXX;
-extern UnaryOp ToAnyX, NegateX, ComplementX, NotX, TrueX;
-extern BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
-extern BinaryOp EqualX, NotEqualX, GreaterX, GreaterEQX;
-extern BinaryOp ShiftLX, ShiftRX, OrX, AndX, XorX;
-extern BinaryOp AssignX, MatMultX;
-extern UnaryOp EvalX, SetupX, PrintX;
-extern MemberOp GetMemberX;
-
 static UnaryOp PrintDBG;
 
 extern Operations debugOps;
@@ -944,8 +935,8 @@ void Y_dbinfo(int nArgs)
     PushDataBlock(NewArray(&stringStruct, (Dimension *)0));
   }
 
-  op.value= dbg->func;   /* PrintXFN only uses this member of op */
-  PrintXFN(&op);
+  op.value= dbg->func;   /* PrintFN only uses this member of op */
+  PrintFN(&op);
   ForceNewline();
   sprintf(lineBuf, "Current debug level is: %d", yDebugLevel);
   PrintFunc(lineBuf);
