@@ -1,8 +1,13 @@
 /*
-    FNCTN.C
-
-    $Id: fnctn.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: fnctn.c,v 1.2 2005/11/13 21:01:56 dhmunro Exp $
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
+ */
 
+/*
     EvalFN evaluates a Yorick interpretive function.  This is one half
     of the Yorick Eval instruction.  The other half is array indexing,
     which is handled in the array.c file.
@@ -41,8 +46,6 @@
        k4 -> p4
 
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 
@@ -271,13 +274,13 @@ void EvalFN(Operand *op)
 
   /* Be sure the stack is long enough for a worst-case invocation of this
      function.  nReq= nPos + (hasPosList&1) + nKey + nLoc + (deepest stack
-                      required for expression evaluation) + 2
+                      required for expression evaluation) + 10
                       + 1 for return address for this function
      The nPos and nKey terms must be present because they may not be
      actual arguments, and because even if they are supplied they may
      be referenceSyms which must be copied for use during return.
-     The extra 2 is so that builtin procedures are always guaranteed
-     two free stack slots without calling CheckStack.  */
+     The extra 10 is so that builtin procedures are always guaranteed
+     8(+2 for luck) free stack slots without calling CheckStack.  */
   if (CheckStack(nReq)) stack= sp-n;
 
   /* Handle all actual parameters.
