@@ -1,12 +1,13 @@
 /*
-    OPS.C
-
-    Define miscellaneous virtual machine functions.
-
-    $Id: ops.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: ops.c,v 1.4 2006/07/20 04:28:30 dhmunro Exp $
+ *
+ * Define miscellaneous virtual machine functions.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "bcast.h"
 #include "pstdlib.h"
@@ -312,7 +313,8 @@ void Build(void)
         YError("non-conformable arguments to build array operator [...]");
       FreeDimension(buildDims);
       buildDims= Ref(tmpDims);
-      if (buildOps[i].type.base==base || ops->promoteID<=T_COMPLEX) {
+      if (buildOps[i].type.base==base ||
+          ((ops->promoteID<=T_COMPLEX) && (promoteID<=T_COMPLEX))) {
         if (ops->promoteID!=promoteID) needPromote= 1;
         if (ops->promoteID>promoteID) promoteID= ops->promoteID;
       } else {
@@ -568,7 +570,7 @@ void MatMultX(Operand *l, Operand *r)
 void CallShell(void)
 {
   Array *array= (Array *)(pc++)->constant->value.db;
-  if (array->value.q[0]) system(array->value.q[0]);
+  if (array->value.q[0]) p_system(array->value.q[0]);
 }
 
 void Y_system(int nArgs)
@@ -576,7 +578,7 @@ void Y_system(int nArgs)
   char *line;
   if (nArgs!=1) YError("system function takes exactly one argument");
   line= YGetString(sp);
-  if (line && line[0]) system(line);
+  if (line && line[0]) p_system(line);
 }
 
 /* ------------------------------------------------------------------------ */
