@@ -1,14 +1,15 @@
 /*
-    OPS0.C
-    Implement type conversion and promotion operations:
-
-    ToChar, ToShort, ToInt, ToLong, ToFloat, ToDouble, ToComplex
-    Promote
-
-    $Id: ops0.c,v 1.1 1993/08/27 18:32:09 munro Exp munro $
+ * $Id: ops0.c,v 1.1.1.1 2005/09/18 22:04:07 dhmunro Exp $
+ * Implement type conversion and promotion operations:
+ *
+ *  ToChar, ToShort, ToInt, ToLong, ToFloat, ToDouble, ToComplex
+ *  Promote
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 
@@ -393,7 +394,7 @@ CONV_Z_TYPE(ToDoubleZ, double, d, doubleStruct, PopToD)
    Non-numeric types return l->ops if the l->ops==r->ops, else 0.  */
 /* typedef Operations *PromoteOp(Operand *l, Operand *r); */
 
-extern PromoteOp PromNOP, PromXX;
+extern PromoteOp PromNOP;
 extern PromoteOp         PromCS, PromCI, PromCL, PromCF, PromCD, PromCZ;
 extern PromoteOp PromSC,         PromSI, PromSL, PromSF, PromSD, PromSZ;
 extern PromoteOp PromIC, PromIS,         PromIL, PromIF, PromID, PromIZ;
