@@ -1,12 +1,13 @@
 /*
-    PARSE.C
-
-    Define functions required to parse Yorick grammar.
-
-    $Id: parse.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: parse.c,v 1.1.1.1 2005/09/18 22:04:12 dhmunro Exp $
+ *
+ * Define functions required to parse Yorick grammar.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "parse.h"
 
@@ -23,8 +24,6 @@ extern int YpReCompare(Function *func,
                        int nLocal, long hasPL, int maxStackDepth,
                        Instruction *code, long codeSize);
 
-extern char *YpLitName(Literal);  /* hook for ScanForFunc in yorick.y */
-
 /* range functions */
 extern RangeFunc RFmin, RFmax, RFptp, RFsum, RFavg, RFrms, RFmnx, RFmxx,
   RFpsum, RFdif, RFzcen, RFpcen, RFuncp, RFcum;
