@@ -1,13 +1,14 @@
 /*
-    STD0.C
-    Define various standard Yorick built-in functions declared in std.i
-
-    See std.i for documentation on the functions defined here.
-
-    $Id: std0.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: std0.c,v 1.7 2007/03/28 09:10:37 thiebaut Exp $
+ * Define various standard Yorick built-in functions declared in std.i
+ *
+ *  See std.i for documentation on the functions defined here.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "bcast.h"
 #include "yio.h"
@@ -37,7 +38,7 @@ extern BuiltIn Y_yorick_init, Y_set_path, Y_reshape, Y_array, Y_structof,
   Y_min, Y_max, Y_sum, Y_avg, Y_allof, Y_anyof, Y_noneof, Y_nallof, Y_where,
   Y_get_cwd, Y_get_home, Y_cd, Y_get_env, Y_get_argv, Y_use_origins;
 
-extern BuiltIn Y_set_site, Y_timer;
+extern BuiltIn Y_set_site, Y_timer, Y_get_path, Y_get_pkgnames;
 
 /*--------------------------------------------------------------------------*/
 
@@ -53,18 +54,12 @@ extern char **ym_argv;
 
 /*--------------------------------------------------------------------------*/
 
-static void YInit(BuiltIn **builtin, void **global,
-                  char **name, char **include);
+static void YInit(char *home, char *site, y_pkg_t **pkgs);
 
 extern int YGetLaunchDir(char *argv0);  /* sysdep.c */
-extern char *yLaunchDir;
-char *yLaunchDir= 0;
 
-/* Codger-generated arrays from ycode.c used in call to YorickInit */
-extern BuiltIn *yBuiltIns[];
-extern void *yGlobals[];
-extern char *yLinkNames[];
-extern char *initialIncludes[];
+char *yLaunchDir=0, *ySiteDir=0, *yHomeDir=0;
+static char *defaultPath = 0;
 
 extern int y_on_idle(void);
 extern void y_on_stdin(char *input_line);
@@ -79,12 +74,13 @@ static void *y_mmfail(unsigned long n);
 static void y_on_keyline(char *msg);
 
 int
-on_launch(int argc, char *argv[])
+y_launch(int argc, char *argv[],
+         char *home, char *site, y_pkg_t **pkgs)
 {
   ym_argc= argc;
   ym_argv= argv;
   p_mmfail = y_mmfail;
-  YInit(yBuiltIns, yGlobals, yLinkNames, initialIncludes);
+  YInit(home, site, pkgs);
   if (!yBatchMode) {
     p_stdinit(y_on_stdin);
     g_on_keyline = y_on_keyline;
@@ -117,51 +113,42 @@ y_mmfail(unsigned long n)
 void Y_yorick_init(int nArgs)
 {
   Drop(nArgs);
-  YInit(yBuiltIns, yGlobals, yLinkNames, initialIncludes);
+  y_pkg_link(0);
 }
 
-static char *defaultPath;
-extern char *yUserPath;   /* defined in ycode.c */
-
-static void YInit(BuiltIn **builtin, void **global,
-                  char **name, char **include)
+static void
+YInit(char *home, char *site, y_pkg_t **pkgs)
 {
-  long index;
-  DataBlock *oldDB;
-
-  /* initialize Yorick timer */
+  /* initialize Yorick timer and user HOME directory */
   Y_timer(-1);
+  YGetHOME();
 
   /* initialize basic data types */
   InitStructDefs();                /* opsv.c */
 
-  /* initialize built-in functions */
-  while (*builtin) {
-    index= Globalize(*name++, 0L);
-    oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-    if (!oldDB || oldDB->ops!=&builtinOps ||
-        ((BIFunction *)oldDB)->function!=*builtin) {
-      globTab[index].value.db= (DataBlock *)NewBIFunction(*builtin, index);
-      globTab[index].ops= &dataBlockSym;
-      Unref(oldDB);
-    }
-    builtin++;
-  }
+  /* add all statically linked packages, and link interpreter to them */
+  while (pkgs[0]) y_pkg_add(*pkgs++);
+  y_pkg_link(0);
 
-  /* initialize compiled variables */
-  while (*global) {
-    index= Globalize(*name++, 0L);
-    oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-    globTab[index].value.db=
-      (DataBlock *)NewLValueM((Array *)0, *global++,
-                              &charStruct, (Dimension *)0);
-    /* note: everything starts out as a scalar char at the correct address
-             -- the reshape operation when the associated package include
-             file is read (below) will set the correct data type and
-             dimension list */
-    globTab[index].ops= &dataBlockSym;
-    Unref(oldDB);
+  /* set startup include file search path:
+        yLaunchDir:ySiteDir/i0:ySiteDir/contrib
+
+     the switch to the normal include file search path is done in
+     stdx.i, which is included just before custom.i
+     -- create the default path here:
+        .:~/.yorick:~/yorick:~/Yorick:ySiteDir/i:ySiteDir/contrib  */
+  yLaunchDir = p_strcpy(ym_argc>0? ym_argv[0] : 0);
+  if (yLaunchDir) {
+    int i;
+    for (i=0 ; yLaunchDir[i] ; i++);
+    while (i>0 && yLaunchDir[i]!='/') i--;
+    if (yLaunchDir[i]=='/') yLaunchDir[i+1] = '\0';
   }
+  ySiteDir = site? p_strcpy(site) : p_strcpy("/");
+  YNameToHead(&ySiteDir);
+  yHomeDir = home? p_strcpy(home) : p_strcpy("/");
+  YNameToHead(&yHomeDir);
+  Y_set_site(0);
 
   /* set up to read user customizations
      If the first two arguments are -batch alternate.i, use alternate.i
@@ -180,31 +167,14 @@ static void YInit(BuiltIn **builtin, void **global,
     }
   }
 
-  /* set up to read std.i and package include files */
-  while (*include) YpPush(*include++);
+  /* final include file is always stdx.i, which returns to defaultPath */
+  YpPush("stdx.i");
 
-  /* first file read is always paths.i, which may override ySiteDir */
+  /* next include all the statically loaded packages */
+  y_pkg_include(0, 0);
+
+  /* first include file is always paths.i, which may override ySiteDir */
   YpPush("paths.i");
-
-  /* set startup include file search path:
-        yLaunchDir:ySiteDir/i0:ySiteDir/contrib
-
-     the switch to the normal include file search path is done in
-     stdx.i, which is included just before custom.i
-     -- create the default path here:
-        .:~/yorick:~/Yorick:ySiteDir/i:ySiteDir/contrib  */
-  yLaunchDir = p_strcpy(ym_argc>0? ym_argv[0] : 0);
-  if (yLaunchDir) {
-    int i;
-    for (i=0 ; yLaunchDir[i] ; i++);
-    while (i>0 && yLaunchDir[i]!='/') i--;
-    if (yLaunchDir[i]=='/') yLaunchDir[i+1] = '\0';
-  }
-  ySiteDir = ySiteDir? p_strcpy(ySiteDir) : p_strcpy("/");
-  YNameToHead(&ySiteDir);
-  Y_set_site(0);
-
-  /* SetSignals(signalFlags); */
 }
 
 void Y_set_path(int nArgs)
@@ -216,11 +186,49 @@ void Y_set_path(int nArgs)
   YpSetPaths(path);
 }
 
-void Y_set_site(int nArgs)
+void Y_get_path(int nArgs)
+{
+  char *path, *p;
+  Array *rslt;
+  long i, len = 1;
+  if (nArgs!=1 || !yarg_nil(0)) YError("get_path takes one nil argument");
+  for (i=0 ; i<nYpPrefixes ; i++) len += strlen(ypPrefixes[i])+1;
+  rslt = (Array *)PushDataBlock(NewArray(&stringStruct, (Dimension *)0));
+  rslt->value.q[0] = path = p_malloc(len);
+  for (i=0 ; i<nYpPrefixes ; i++) {
+    p = ypPrefixes[i];
+    while (p[0]) *path++ = *p++;
+    *path++ = ':';
+  }
+  if (nYpPrefixes) path--;
+  path[0] = '\0';
+}
+
+void
+Y_get_pkgnames(int nargs)
+{
+  int i, npkg;
+  Array *rslt;
+  Dimension *tmp = tmpDims;
+  if (nargs!=1) YError("get_pkgnames takes exactly one argument");
+  npkg = y_pkg_count(yarg_si(0));
+  if (npkg<=0) YError("get_pkgnames -- no packages loaded yet");
+  tmpDims = 0;
+  FreeDimension(tmp);
+  tmpDims = NewDimension((long)npkg, 1L, (Dimension *)0);
+  rslt = (Array *)PushDataBlock(NewArray(&stringStruct, tmpDims));
+  for (i=0 ; i<npkg ; i++)
+    rslt->value.q[i] = p_strcpy(y_pkg_name(i));
+}
+
+static char *y_make_ipath(char *ylaunch, char *ysite, char *yhome);
+
+void
+Y_set_site(int nArgs)
 {
   /* This function should ONLY be called from paths.i, or by YInit  */
 
-  char *path, *pathtmp, *site;
+  char *path;
   Array *dirName;
   DataBlock *oldDB;
   long index;
@@ -228,23 +236,27 @@ void Y_set_site(int nArgs)
   if (nArgs>2) YError("expecting zero, one, or two arguments");
 
   /* record Y_LAUNCH in global variable */
-  index= Globalize("Y_LAUNCH", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
-  globTab[index].ops= &dataBlockSym;
+  index = Globalize("Y_LAUNCH", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
+  globTab[index].ops = &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(yLaunchDir);
+  dirName->value.q[0] = p_strcpy(yLaunchDir);
 
   if (nArgs>=1) {
-    path= YGetString(sp-nArgs+1);
-    if (nArgs==2) yHomeDir= YGetString(sp);
+    ySiteDir = p_strcpy(YGetString(sp-nArgs+1));
+    YNameToHead(&ySiteDir);
+    if (nArgs==2) {
+      yHomeDir = p_strcpy(YGetString(sp));
+      YNameToHead(&yHomeDir);
+    }
   } else {
 #ifndef NO_FALLBACK_TEST
     p_file *test = 0;
-    if (ySiteDir && ySiteDir[0]) {
-      path= p_strncat(ySiteDir, "i0/std.i", 0);
-      test= p_fopen(path, "r");
+    if (ySiteDir && ySiteDir[0] && YIsAbsolute(ySiteDir)) {
+      path = p_strncat(ySiteDir, "i0/std.i", 0);
+      test = p_fopen(path, "r");
       p_free(path);
     }
     if (test) {
@@ -256,14 +268,14 @@ void Y_set_site(int nArgs)
         for (i-=2 ; i>0 ; i-=2)
           if (yLaunchDir[i-1]!='/' || yLaunchDir[i]!='.') break;
         for (; i>0 ; i--) if (yLaunchDir[i]=='/') break;
-        path = p_malloc(i+9);
+        path = p_malloc(i+10);
         strncpy(path, yLaunchDir, i+1);
         strcpy(path+i+1, "i0/std.i");
         test= p_fopen(path, "r");
         if (test) {
           p_fclose(test);
           path[i+1] = '\0';
-          ySiteDir= path;
+          ySiteDir = path;
           /* if ySiteDir was bad, so is yHomeDir, guess the same */
           yHomeDir = p_strcpy(ySiteDir);  
         } else {
@@ -272,85 +284,83 @@ void Y_set_site(int nArgs)
       }
     }
 #endif
-    path= ySiteDir;
   }
 
   /* record Y_VERSION in global variable */
-  index= Globalize("Y_VERSION", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
-  globTab[index].ops= &dataBlockSym;
+  index = Globalize("Y_VERSION", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
+  globTab[index].ops = &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(yVersion);
+  dirName->value.q[0] = p_strcpy(yVersion);
 
   /* record Y_HOME in global variable */
-  index= Globalize("Y_HOME", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
-  globTab[index].ops= &dataBlockSym;
+  index = Globalize("Y_HOME", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
+  globTab[index].ops = &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(yHomeDir);
-  YNameToHead(dirName->value.q);
+  dirName->value.q[0] = p_strcpy(yHomeDir);
 
   /* record Y_SITE in global variable */
-  index= Globalize("Y_SITE", 0L);
-  oldDB= globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
-  dirName= NewArray(&stringStruct, (Dimension *)0);
-  globTab[index].value.db= (DataBlock *)dirName;
+  index = Globalize("Y_SITE", 0L);
+  oldDB = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+  dirName = NewArray(&stringStruct, (Dimension *)0);
+  globTab[index].value.db = (DataBlock *)dirName;
   globTab[index].ops= &dataBlockSym;
   Unref(oldDB);
-  dirName->value.q[0]= p_strcpy(path);
+  dirName->value.q[0] = p_strcpy(ySiteDir);
   YNameToHead(dirName->value.q);
-  site= dirName->value.q[0];
 
-  /* construct startup include file search path:
-        yLaunchDir:ySiteDir/i0:ySiteDir/contrib        */
-  if (yLaunchDir && strlen(yLaunchDir) &&
-      strcmp(yLaunchDir, site)) pathtmp= p_strncat(yLaunchDir, PATH_SEP, 0);
-  else pathtmp= 0;
-  path= p_strncat(pathtmp, site, 0);
-  p_free(pathtmp);
-  pathtmp= p_strncat(path, "i0", 0);
-  p_free(path);
-  path= p_strncat(pathtmp, PATH_SEP, 0);
-  p_free(pathtmp);
-  pathtmp= p_strncat(path, site, 0);
-  p_free(path);
-  path= p_strncat(pathtmp, "contrib", 0);
-  p_free(pathtmp);
-
-  /* set the initial include path */
+  /* set the initial include path for paths.i */
+  path = y_make_ipath(yLaunchDir, ySiteDir, yHomeDir);
   YpSetPaths(path);
   p_free(path);
 
-  /* the switch to the normal include file search path is done in
-     stdx.i, which is included just before custom.i
-     -- create the default path here:
-        .:~/yorick:~/Yorick:ySiteDir/i:ySiteDir/contrib  */
-  if (yUserPath)
-    path= p_strncat(yUserPath, PATH_SEP, 0);
-  else
-    path= 0;
-  /* if (yLaunchDir && yLaunchDir[0] &&
-   *     strcmp(yLaunchDir, site)) {
-   *   pathtmp= p_strncat(path, yLaunchDir, 0);
-   *   p_free(path);
-   *   path= p_strncat(pathtmp, "i" PATH_SEP, 0);
-   *   p_free(pathtmp);
-   * }
-   */
-  pathtmp= p_strncat(path, site, 0);
-  p_free(path);
-  path= p_strncat(pathtmp, "i" PATH_SEP, 0);
-  p_free(pathtmp);
-  pathtmp= p_strncat(path, site, 0);
-  p_free(path);
-  path= p_strncat(pathtmp, "contrib", 0);
-  p_free(pathtmp);
+  defaultPath = y_make_ipath(0, ySiteDir, yHomeDir);
+}
 
-  defaultPath= path;
+static char *
+y_make_ipath(char *ylaunch, char *ysite, char *yhome)
+{
+  /*~/.yorick:~/yorick:~/Yorick:Y_SITE/i:Y_SITE/contrib:Y_SITE/i0:Y_HOME/lib*/
+  char *path1 = 0, *path2 = 0;
+  char *yuser = "~/.yorick:";
+  p_dir *yudir = p_dopen("~/yorick");
+  if (yudir) yuser = "~/yorick:";
+  else if ((yudir = p_dopen("~/Yorick"))) yuser = "~/Yorick:";
+  if (yudir) p_dclose(yudir);
+  if (ylaunch && ylaunch[0]) {
+    /* prepend Y_LAUNCH for paths.i only */
+    long len = strlen(ylaunch);
+    path1 = (ylaunch[len-1]=='/')? p_strncat(0, ylaunch, len-1) :
+      p_strcpy(ylaunch);
+    path2 = p_strncat(path1, PATH_SEP, 0);
+    p_free(path1);
+  } else {
+    path2 = p_strcpy("." PATH_SEP);
+  }
+  path1 = p_strncat(path2, yuser, 0);
+  if (path2) p_free(path2);
+  path2 = p_strncat(path1, ysite, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "i" PATH_SEP, 0);
+  p_free(path2);
+  path2 = p_strncat(path1, ysite, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "contrib" PATH_SEP, 0);
+  p_free(path2);
+  path2 = p_strncat(path1, ysite, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "i0" PATH_SEP, 0);
+  p_free(path2);
+  path2 = p_strncat(path1, yhome, 0);
+  p_free(path1);
+  path1 = p_strncat(path2, "lib", 0);
+  p_free(path2);
+  return path1;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -727,6 +737,7 @@ void Y_nameof(int nArgs)
     long index;
     if (db->ops==&functionOps) index= ((Function *)db)->code[0].index;
     else if (db->ops==&builtinOps) index= ((BIFunction *)db)->index;
+    else if (db->ops==&auto_ops) index= ((autoload_t *)db)->isymbol;
     else index= -1;
     if (index>=0) name= globalTable.names[index];
     else name= 0;
@@ -775,6 +786,7 @@ void Y_is_func(int nArgs)
       Operations *ops= s->value.db->ops;
       if (ops==&functionOps) isFunc= 1;
       else if (ops==&builtinOps) isFunc= 2;
+      else if (ops==&auto_ops) isFunc= 3;
       else isFunc= 0;
       break;
     } else if (s->ops!=&referenceSym) {
@@ -796,7 +808,7 @@ void Y_is_void(int nArgs)
   for (;;) {
     if (s->ops==&dataBlockSym) {
       Operations *ops= s->value.db->ops;
-      isVoid= (ops==&voidOps);
+      isVoid= (ops==&voidOps || ops==&auto_ops);
       break;
     } else if (s->ops!=&referenceSym) {
       isVoid= 0;
@@ -920,17 +932,20 @@ static void UnaryTemplate(int nArgs, Looper *DLooper, Looper *ZLooper)
   if (nArgs != 1) YError("expecting exactly one argument");
   sp->ops->FormOperand(sp, &op);
   promoteID= op.ops->promoteID;
-  errno = 0;
   if (promoteID<=T_DOUBLE) {
     if (promoteID<T_DOUBLE) op.ops->ToDouble(&op);
+    errno = 0;
     DLooper(BuildResultU(&op, &doubleStruct), op.value, op.type.number);
+    promoteID = errno;
     PopToD(sp-2);
   } else {
     if (promoteID>T_COMPLEX) YError("expecting numeric argument");
+    errno = 0;
     ZLooper(BuildResultU(&op, &complexStruct), op.value, 2*op.type.number);
+    promoteID = errno;
     PopTo(sp-2);
   }
-  if (errno) YError("mathlib function signals error");
+  if (promoteID) YError("mathlib function signals error");
   Drop(1);
 }
 
@@ -1494,8 +1509,9 @@ void Y_sum(int nArgs)
   int promoteID;
   if (nArgs != 1) YError("sum requires exactly one argument");
   sp->ops->FormOperand(sp, &op);
-  promoteID= op.ops->promoteID;
-  if (promoteID>T_COMPLEX) YError("sum requires numeric argument");
+  promoteID= (op.ops->typeID!=T_STRING)? op.ops->promoteID : T_STRING;
+  if (promoteID>T_COMPLEX && promoteID!=T_STRING)
+    YError("sum requires numeric argument");
   array= Force1D(&op);
   if (!array) {                 /* scalars are trivial */
     PopTo(sp-1);
@@ -1503,7 +1519,7 @@ void Y_sum(int nArgs)
   }
   RFsum(array, 0);
   /* sum range function may change data type */
-  PopToX(sp-2, sp->value.db->ops->promoteID);
+  PopToX(sp-2, (promoteID!=T_STRING)?sp->value.db->ops->promoteID:T_STRING);
   Drop(1);
 }
 
@@ -1716,15 +1732,6 @@ struct OriginStatus {
 extern OriginStatus *NewOrg(int force);
 extern void FreeOrg(void *ob);  /* ******* Use Unref(bm) ******* */
 
-extern PromoteOp PromXX;
-extern UnaryOp ToAnyX, NegateX, ComplementX, NotX, TrueX;
-extern BinaryOp AddX, SubtractX, MultiplyX, DivideX, ModuloX, PowerX;
-extern BinaryOp EqualX, NotEqualX, GreaterX, GreaterEQX;
-extern BinaryOp ShiftLX, ShiftRX, OrX, AndX, XorX;
-extern BinaryOp AssignX, MatMultX;
-extern UnaryOp EvalX, SetupX, PrintX;
-extern MemberOp GetMemberX;
-
 static UnaryOp PrintOB;
 
 Operations orgsOps = {
