@@ -1,11 +1,12 @@
 /*
-   STYLE.C
-   Set/get details of graphics style for style.i functions.
-
-   $Id$
+ * $Id: style.c,v 1.2 2007/12/26 04:25:46 dhmunro Exp $
+ * Set/get details of graphics style for style.i functions.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1996.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "gist.h"
 #include "hlevel.h"
@@ -43,8 +44,7 @@ extern BuiltIn Y_raw_style;
 void Y_raw_style(int nArgs)
 {
   if (nArgs!=4) YError("raw_style takes exactly 4 arguments");
-  PushIntValue(raw_style(YGetInteger(sp-3),YGet_I(sp-2,0,0),
-                         *YGet_P(sp-1,0,0),*YGet_P(sp,0,0)));
+  PushIntValue(raw_style(yarg_si(3),yarg_i(2,0),*yarg_p(1,0),*yarg_p(0,0)));
 }
 
 int raw_style(long nsys, int *landscape,
@@ -141,6 +141,7 @@ void Y_viewport(int nArgs)
     PushDataBlock(NewArray(&doubleStruct,
                            NewDimension(4L, 1L, (Dimension *)0)));
   double *port= array->value.d;
+  array->type.dims->references--;
   port[0]= gistD.trans.viewport.xmin;
   port[1]= gistD.trans.viewport.xmax;
   port[2]= gistD.trans.viewport.ymin;
