@@ -1,11 +1,12 @@
 /*
-    TASK.C
-    Implement Yorick virtual machine.
-
-    $Id: task.c,v 1.1 1993/08/27 18:32:09 munro Exp munro $
+ * $Id: task.c,v 1.7 2007/02/24 23:14:41 dhmunro Exp $
+ * Implement Yorick virtual machine.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "ydata.h"
 #include "yio.h"
@@ -32,13 +33,13 @@ extern int YpParse(void *func);  /* argument non-zero for YpReparse */
 /*--------------------------------------------------------------------------*/
 
 extern BuiltIn Y_quit, Y_include, Y_require, Y_help, Y_exit, Y_error, Y_batch;
+extern BuiltIn Y_plug_in, Y_plug_dir, Y_maybe_prompt, Y_suspend, Y_resume;
+extern BuiltIn Y_after;
 
 extern void YRun(void);
 extern void YHalt(void);
 
 extern int CheckForTasks(int wait);
-extern void IncludeNow(void);
-extern void ResetStack(int hard);
 
 extern VMaction Eval, Return, PushVariable;
 
@@ -55,8 +56,8 @@ int yAutoDebug= 0;
 int yDebugLevel= 0;
 
 /* most recent error message was built in yErrorMsg */
-char yErrorMsg[132];
-char yWarningMsg[132];
+char yErrorMsg[192];
+char yWarningMsg[192];
 
 static int inYError= 0;
 
@@ -97,8 +98,8 @@ static int caughtTask= 0;
 
 /* stuff to implement set_idler */
 extern BuiltIn Y_set_idler;
-extern Function *y_idler_function;
 Function *y_idler_function= 0;
+static int y_idler_fault = 0;
 
 /*--------------------------------------------------------------------------*/
 
@@ -155,7 +156,6 @@ YHalt(void)
 }
 
 /* if read() interrupted by uncaught error, have a serious problem */
-extern void yr_reset(void);
 extern char *y_read_prompt;
 extern Instruction *ym_suspend(void);
 extern void ym_resume(Instruction *pc);
@@ -231,6 +231,9 @@ PushTask(Function *task)
 static Instruction taskCode[4];
 static int taskCodeInit= 0;
 
+static Instruction *ym_suspc = 0;
+extern int yg_blocking;         /* graph.c */
+
 int DoTask(void)
 {
   extern Operations debugOps;
@@ -249,6 +252,14 @@ int DoTask(void)
     pc= taskCode;     /* note that original pc is clobbered */
   }
   YRun();
+  if (yg_blocking == 4) {  /* resume has been called */
+    yg_blocking = 0;
+    if (ym_suspc) {
+      Instruction *suspc = ym_suspc;
+      ym_suspc = 0;
+      ym_resume(suspc);
+    }
+  }
   if (ym_state & Y_PENDING) {
     ym_state &= ~Y_PENDING;
     ym_state |= Y_SUSPENDED;
@@ -278,7 +289,6 @@ static int y_was_idle= 0;
 
 static void ym_prompter(void);
 extern char *y_read_prompt;     /* ascio.c */
-extern int yg_blocking;         /* graph.c */
 /* yp_did_prompt reset by y_on_stdin */
 extern int yp_did_prompt;
 int yp_did_prompt= 0;
@@ -302,11 +312,65 @@ ym_prompter(void)
   }
 }
 
+void
+Y_maybe_prompt(int nargs)
+{
+  if (nargs) YError("maybe_prompt accepts no arguments");
+  if (!yp_did_prompt && !y_read_prompt) ym_prompter();
+}
+
+/* suspend/resume is very primitive - would be nice to permit a
+ * queue of suspended tasks, but needs a bigger interpreted API
+ */
+void
+Y_suspend(int nargs)
+{
+  if (ym_suspc || yg_blocking || y_read_prompt)
+    YError("suspend: already suspended, paused, or waiting for input");
+  ym_suspc = ym_suspend();
+  yg_blocking = 3;     /* for y_on_stdin */
+}
+
+void
+Y_resume(int nargs)
+{
+  /* cannot call ym_resume directly, signal DoTask to do that */
+  if (ym_suspc && yg_blocking==3) yg_blocking = 4;
+}
+
+static void ym_after(void *context);
+
+void
+Y_after(int nargs)
+{
+  double secs;
+  Operand op, *pop;
+  long callout = -1;
+  if (nargs != 2) YError("after: accepts precisely two arguments");
+  secs = yarg_sd(nargs-1);
+  pop = yarg_op(nargs-2, &op);
+  if (!pop || op.ops!=&functionOps)
+    YError("after: second argument must be interpreted callback function");
+  callout = ((Function *)op.value)->code[0].index;
+  p_set_alarm(secs, ym_after, callout+(char*)0);
+}
+
+static void
+ym_after(void *context)
+{
+  long index = (char *)context - (char*)0;
+  DataBlock *db = globTab[index].ops==&dataBlockSym?
+    globTab[index].value.db : 0;
+  if (!db || db->ops!=&functionOps) YError("after callback function lost");
+  else PushTask((Function*)Ref(db));
+}
+
 int
 y_on_idle(void)
 {
   int more_work = 0;
   int pending_stdin = 0;
+  int idler_fault = 0;
 
   if (!taskCodeInit) {
     taskCode[0].Action = &Eval;
@@ -332,6 +396,11 @@ y_on_idle(void)
           y_idler_function) {
         Function *f = y_idler_function;
         y_idler_function = 0;
+        /* this does not really detect when the after_error function
+         * itself has completed - a catch inside after_error can get
+         * complicated and probably defeat this loop detection attempt
+         */
+        idler_fault = y_idler_fault;
         PushTask(f);
         Unref(f);
       }
@@ -340,6 +409,7 @@ y_on_idle(void)
 
   if (nTasks+caughtTask) DoTask();
   if (ym_state & Y_QUITTING) goto die_now;
+  if (idler_fault) y_idler_fault = 0;
   more_work = (nTasks+caughtTask || nYpIncludes || nYpInputs ||
                pending_stdin || y_idler_function);
 
@@ -455,25 +525,270 @@ void IncludeNow(void)
   if (ym_state&Y_QUITTING) YHalt();
 }
 
-void Y_include(int nArgs)
+void
+Y_include(int nArgs)
 {
   long now;
   char *name;
   if (nArgs!=1 && nArgs!=2)
     YError("include function takes exactly one or two arguments");
-  if (nArgs>1) now= YGetInteger(sp);
-  else now= 0;
-  name= YGetString(sp-nArgs+1);
+  now = (nArgs>1)? YGetInteger(sp) : 0;
+  name = YGetString(sp-nArgs+1);
   if (name[0]) {  /* name=="" special hack cleans out pending includes */
-    if (now>=0 && !YpPushInclude(name))
-      YError("missing include file specified in include function");
-    else if (now<0)
+    if (now >= 0) {
+      if (!YpPushInclude(name)) {
+        if (!(now&2))
+          YError("missing include file specified in include function");
+        now = 0;
+      }
+    } else {
       YpPush(name);          /* defer until all pending input parsed */
+    }
   }
   Drop(nArgs);
   if (now>0) IncludeNow(); /* parse and maybe execute file to be included
-                              -- without now, this won't happen until the
-                              next line is parsed naturally */
+                            * -- without now, this won't happen until the
+                            * next line is parsed naturally */
+}
+
+static char **yplug_path = 0;
+
+void
+Y_plug_dir(int nArgs)
+{
+  Dimension *dims = 0;
+  char **d = (nArgs==1)? YGet_Q(sp, 1, &dims) : 0;
+  long nd = d? TotalNumber(dims) : 0;
+  int i;
+  if (nArgs > 1)
+    YError("plug_dir function takes at most one argument");
+  if (nArgs && !CalledAsSubroutine()) {
+    for (i=0 ; yplug_path && yplug_path[i] ; i++);
+    if (i > 1) {
+      int n = i - 1;
+      Array *rslt;
+      Dimension *tmp = tmpDims;
+      tmpDims = 0;
+      FreeDimension(tmp);
+      tmpDims = NewDimension((long)n, 1L, (Dimension *)0);
+      rslt = (Array *)PushDataBlock(NewArray(&stringStruct, tmpDims));
+      for (i=0 ; i<n ; i++)
+        rslt->value.q[i] = p_strcpy(yplug_path[i]);
+    } else {
+      PushDataBlock(RefNC(&nilDB));
+    }
+    if (!nd) return;
+  }
+  if (yplug_path) {
+    for (i=0 ; yplug_path[i] ; i++) {
+      p_free(yplug_path[i]);
+      yplug_path[i] = 0;
+    }
+    p_free(yplug_path);
+    yplug_path = 0;
+  }
+  yplug_path = p_malloc(sizeof(char *)*(nd+2));
+  for (i=0 ; i<=nd ; i++) {
+    if (i < nd) {
+      yplug_path[i] = YExpandName(d[i]);
+      YNameToHead(&yplug_path[i]);
+    } else {
+      yplug_path[i] = p_strncat(yHomeDir, "lib/", 0L);
+    }
+  }
+  yplug_path[nd+1] = 0;
+}
+
+struct y_package_t {
+  char *name;
+  y_pkg_t *init;
+  char **ifiles;
+  BuiltIn **code;
+  void **data;
+  char **varname;
+};
+static struct y_package_t *y_pkg_list = 0;
+static int y_npkg = 0;
+static int y_n0pkg = 0;
+
+char *
+y_pkg_name(int i)
+{
+  return (i<0 || i>=y_npkg || !y_pkg_list)? 0 : y_pkg_list[i].name;
+}
+
+int
+y_pkg_count(int i)
+{
+  return i? y_npkg : y_n0pkg;
+}
+
+y_pkg_t *
+y_pkg_lookup(char *name)
+{
+  int i;
+  for (i=0 ; i<y_npkg ; i++)
+    if (!strcmp(y_pkg_list[i].name, name))
+      return y_pkg_list[i].init;
+  return 0;
+}
+
+void
+y_pkg_add(y_pkg_t *init)
+{
+  char **ifiles, **varname;
+  BuiltIn **code;
+  void **data;
+  char *pkgname = init(&ifiles, &code, &data, &varname);
+  if (!y_pkg_list)
+    y_pkg_list = p_malloc(sizeof(struct y_package_t)*8);
+  else if (!(y_npkg & 7))
+    y_pkg_list = p_malloc(sizeof(struct y_package_t)*(y_npkg+8));
+  y_pkg_list[y_npkg].name = pkgname;
+  y_pkg_list[y_npkg].init = init;
+  y_pkg_list[y_npkg].ifiles = ifiles;
+  y_pkg_list[y_npkg].code = code;
+  y_pkg_list[y_npkg].data = data;
+  y_pkg_list[y_npkg].varname = varname;
+  y_npkg++;
+}
+
+static void y_pkg_0link(char **varname, BuiltIn **code, void **data);
+
+void
+y_pkg_link(char *name)
+{
+  /* use name==0 to relink every package */
+  int i;
+  if (!y_n0pkg) y_n0pkg = y_npkg;
+  for (i=0 ; i<y_npkg ; i++) {
+    if (!name || !strcmp(y_pkg_list[i].name, name)) {
+      y_pkg_0link(y_pkg_list[i].varname,
+                  y_pkg_list[i].code, y_pkg_list[i].data);
+      if (name) break;
+    }
+  }
+}
+
+static void
+y_pkg_0link(char **varname, BuiltIn **code, void **data)
+{
+  long index;
+  DataBlock *db;
+
+  /* initialize built-in functions */
+  if (code) while (*code) {
+    index = Globalize(*varname++, 0L);
+    db = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+    if (!db || db->ops!=&builtinOps ||
+        ((BIFunction *)db)->function!=*code) {
+      globTab[index].value.db = (DataBlock *)NewBIFunction(*code, index);
+      globTab[index].ops = &dataBlockSym;
+      Unref(db);
+    }
+    code++;
+  }
+
+  /* initialize compiled variables */
+  if (data) while (*data) {
+    index = Globalize(*varname++, 0L);
+    db = globTab[index].ops==&dataBlockSym? globTab[index].value.db : 0;
+    if (!db || db->ops!=&lvalueOps || ((LValue *)db)->type.base->file ||
+        ((LValue *)db)->owner || ((LValue *)db)->address.m!=*data) {
+      /* note: everything starts out as a scalar char at the correct address
+       * - subsequent reshape will set true data type and dimensions
+       */
+      globTab[index].value.db =
+        (DataBlock *)NewLValueM((Array *)0, *data++,
+                                &charStruct, (Dimension *)0);
+      globTab[index].ops = &dataBlockSym;
+    }
+    Unref(db);
+  }
+}
+
+void
+y_pkg_include(char *name, int now)
+{
+  char **ifiles, *msg;
+  int i;
+  /* use name==0 to relink every package */
+  if (now) {
+    for (i=0 ; i<y_npkg ; i++) {
+      if (!name || !strcmp(y_pkg_list[i].name, name)) {
+        ifiles = y_pkg_list[i].ifiles;
+        if (ifiles) while (ifiles[0]) {
+          if (YpPushInclude(*ifiles++)) {
+            IncludeNow();
+          } else {
+            msg= p_strncat("missing include file ", ifiles[-1], 0);
+            YWarning(msg);
+            p_free(msg);
+          }
+        }
+        if (name) break;
+      }
+    }
+  } else {
+    /* when not immediate (e.g.- at startup) push in reverse order */
+    for (i=y_npkg-1 ; i>=0 ; i--) {
+      if (!name || !strcmp(y_pkg_list[i].name, name)) {
+        ifiles = y_pkg_list[i].ifiles;
+        if (ifiles) {
+          while (ifiles[0]) ifiles++;
+          for (ifiles-- ; ifiles>=y_pkg_list[i].ifiles ; ifiles--)
+            YpPush(ifiles[0]);
+        }
+        if (name) break;
+      }
+    }
+  }
+}
+
+void
+Y_plug_in(int nArgs)
+{
+  char *pname, *pkgname;
+  y_pkg_t *init = 0;
+  void *plug;
+  int i;
+  if (nArgs!=1) YError("plug_in function takes exactly one argument");
+  pkgname = YGetString(sp-nArgs+1);
+  if (!pkgname || !pkgname[0]) 
+    YError("plug_in: package name argument is null");
+  for (i=1 ; pkgname[i] ; i++);
+  for (pname=pkgname+i-1 ; pname>pkgname ; pname--)
+    if (pname[0]=='/' || pname[0]=='\\') break;
+
+  /* become a no-op if pkgname already plugged in */
+  if (y_pkg_lookup(pname)) return;
+
+  if (!yplug_path) Y_plug_dir(0);
+
+  plug = p_dlopen(pkgname);
+  if (!plug && !YIsAbsolute(pkgname)) {
+    /* check in plug_dir and Y_HOME/lib before giving up */
+    char *tmp;
+    for (i=0 ; yplug_path[i] ; i++) {
+      tmp = p_strncat(yplug_path[i], pkgname, 0L);
+      plug = p_dlopen(tmp);
+      p_free(tmp);
+      if (plug) break;
+    }
+  }
+  if (plug) {
+    char *tmp = p_strncat("yk_", pname, 0);
+    int failed = p_dlsym(plug, tmp, 0, &init);
+    p_free(tmp);
+    if (failed || !init)
+      YError("plug_in: dynamic library missing yk_<pkgname> function");
+  }
+  if (init) {
+    y_pkg_add(init);
+    y_pkg_link(pname);
+  } else {
+    YError("plug_in: unable to find dynamic library file");
+  }
 }
 
 void Y_require(int nArgs)
@@ -593,9 +908,13 @@ static char *includeFile= 0;
 static long mainIndex= -1;
 Instruction *yErrorPC= 0;   /* for dbup function in debug.c */
 
+static void yset_catchmsg(char *tmsg);
+static long after_index = -1;
+
 void
 YError(const char *msg)
 {
+  extern void yg_got_expose(void);
   long beginLine= ypBeginLine;
   Instruction *pcDebug= pc;
   Function *func;
@@ -604,6 +923,8 @@ YError(const char *msg)
   Instruction *pcUp= yErrorPC;
   int category;
   int no_abort = y_do_not_abort;
+  int no_print = 0;
+  char tmsg[144];
 
   int recursing= inYError;
   inYError++;
@@ -616,13 +937,6 @@ YError(const char *msg)
   ym_state &= ~Y_PENDING;
   ym_dbenter = 0;
 
-  if (y_idler_function) {
-    /* remove any idler on error - catch can reset if desired */
-    Function *f= y_idler_function;
-    y_idler_function= 0;
-    Unref(f);
-  }
-
   if (recursing>8 || yImpossible>8) {
     YputsErr("****FATAL**** YError looping -- quitting now");
     ym_state|= Y_QUITTING;
@@ -645,6 +959,29 @@ YError(const char *msg)
     YputsErr("****OOPS**** error on read resume or throw/catch");
   }
 
+  if (y_idler_function) {
+    /* remove any idler on error - after_error can reset if desired */
+    Function *f= y_idler_function;
+    y_idler_function= 0;
+    Unref(f);
+  }
+  if (after_index < 0) after_index = yget_global("after_error", 0L);
+  if (!y_idler_fault
+      && globTab[after_index].ops == &dataBlockSym
+      && globTab[after_index].value.db->ops == &functionOps) {
+    /* if after_error function present, make it the idler */
+    y_idler_function = (Function *)Ref(globTab[after_index].value.db);
+    y_idler_fault = 1;
+    if (msg) strncpy(tmsg, msg, 140), tmsg[140] = '\0';
+    else tmsg[0] = '\0';
+  } else {
+    y_idler_fault = 0;
+    tmsg[0] = '\0';
+  }
+
+  /* this is a nasty hack for mpy and after_error */
+  no_print = (msg && !strcmp(msg, ".SYNC."));
+
   func= (((ym_state&Y_RUNNING)||no_abort) && !recursing &&
          pcUp!=&taskCode[2])? FuncContaining(pcDebug) : 0;
   name= func? globalTable.names[func->code[0].index] : "VM idle or lost";
@@ -689,8 +1026,8 @@ YError(const char *msg)
   strncat(yErrorMsg, name, 40);
   strcat(yErrorMsg, ") ");
   if (!pcUp || recursing)
-    strncat(yErrorMsg, msg, 80);
-  YputsErr(yErrorMsg);
+    strncat(yErrorMsg, msg, 140);
+  if (!no_print) YputsErr(yErrorMsg);
 
   if (recursing) {
     func= 0;
@@ -699,11 +1036,15 @@ YError(const char *msg)
     func= 0;
     if (y_read_prompt)
       YputsErr("WARNING aborting on YError during keyboard read()");
+    else if (ym_suspc)
+      YputsErr("WARNING aborting on YError during suspend");
     else
       YputsErr("WARNING aborting on YError after mouse() pause() or wait=1");
   }
+  ym_suspc = 0;
+  if (yg_blocking==3 || yg_blocking==4) yg_blocking = 0;
 
-  if (func) {
+  if (func && !no_print) {
     /* Try to find the source code for this function.  */
     long index= func->code[0].index;
     if (mainIndex<0) mainIndex= Globalize("*main*", 0L);
@@ -740,7 +1081,13 @@ YError(const char *msg)
       YputsErr("Function corrupted, cannot enter debug mode.");
     }
 
-    if (!yBatchMode && !pcUp && (!yAutoDebug || yDebugLevel>1)) {
+    if (y_idler_function) {
+      /* special after_error function will get control */
+      ResetStack(1);
+      yr_reset();
+      yg_got_expose();
+      p_clr_alarm(0, 0);
+    } else if (!yBatchMode && !pcUp && (!yAutoDebug || yDebugLevel>1)) {
       if (yDebugLevel>1) {
         YputsErr(" To enter recursive debug level, type <RETURN> now");
       } else {
@@ -751,11 +1098,11 @@ YError(const char *msg)
     }
 
   } else {
-    extern void yg_got_expose(void);
     /* Clear the stack back to the most recent debugging level,
      * or completely clear if aborting a read() operation.  */
-    if (recursing<5) ResetStack(y_read_prompt!=0);
-    else {
+    if (recursing<5) {
+      ResetStack(y_read_prompt!=0);
+    } else {
       YputsErr("****SEVERE**** YError unable to reset stack -- "
                "probably lost variables");
       sp= spBottom;
@@ -772,7 +1119,7 @@ YError(const char *msg)
     return;
   }
 
-  if (yBatchMode || !strncmp(msg,"(FATAL)",7)) {
+  if ((yBatchMode && !y_idler_function) || !strncmp(msg,"(FATAL)",7)) {
     if (yBatchMode) YputsErr("yorick: quitting on error in batch mode");
     else YputsErr("yorick: quitting on fatal error");
     ym_fatal = yBatchMode? 1 : 2;
@@ -780,6 +1127,9 @@ YError(const char *msg)
     ym_state|= Y_QUITTING;
   }
 
+  /* set catch_message variable for after_error function */
+  if (y_idler_function) yset_catchmsg(tmsg);
+
   /* Go back to the main loop.  */
   inYError= 0;
   if (!no_abort) p_abort();
@@ -879,6 +1229,7 @@ void Y_error(int nArgs)
   else YError("<interpreted error function called>");
 }
 
+/* FIXME: this shold also turn off on_stdin event handling */
 void Y_batch(int nArgs)
 {
   int flag= 2;
@@ -932,37 +1283,28 @@ static Catcher *CatchScan(const char *msg, int category)
   }
 
   if (i>=0) {
-    char tmsg[84];
-    Array *array;
-    long cmsg;
+    char tmsg[144];
     Instruction *pcRet;
     Symbol *spCatch= spBottom + catchers[i].isp;
     catchers[i].category= 0;  /* disable this catcher */
 
     /* note: msg itself might be on stack! */
-    strncpy(tmsg, msg, 80);
-    tmsg[80]= '\0';
+    strncpy(tmsg, msg, 140);
+    tmsg[140]= '\0';
 
     for (;;) {
       ClearStack();
       if (spCatch >= sp) break;
       pcRet= AbortReturn();
-      if (!pcRet || pcRet==&taskCode[2]) YError("(BUG) impossible catch");
+      if (!pcRet || pcRet==&taskCode[2])
+        YError("catch does not work outside immediate include or require");
     }
     if (spCatch!=sp) YError("(BUG) impossible catch or corrupt stack");
     pc= catchers[i].pc;
     PushIntValue(1);
 
     /* set catch_message variable (after stack cleared) */
-    cmsg= Globalize("catch_message", 0L);
-    if (globTab[cmsg].ops==&dataBlockSym) {
-      globTab[cmsg].ops= &intScalar;
-      Unref(globTab[cmsg].value.db);
-    }
-    array= NewArray(&stringStruct, (Dimension *)0);
-    globTab[cmsg].value.db= (DataBlock *)array;
-    globTab[cmsg].ops= &dataBlockSym;
-    array->value.q[0]= p_strcpy(tmsg);
+    yset_catchmsg(tmsg);
 
     return &catchers[i];
 
@@ -971,6 +1313,21 @@ static Catcher *CatchScan(const char *msg, int category)
   }
 }
 
+static void
+yset_catchmsg(char *tmsg)
+{
+  Array *array;
+  long cmsg = Globalize("catch_message", 0L);
+  if (globTab[cmsg].ops==&dataBlockSym) {
+    globTab[cmsg].ops = &intScalar;
+    Unref(globTab[cmsg].value.db);
+  }
+  array=  NewArray(&stringStruct, (Dimension *)0);
+  globTab[cmsg].value.db = (DataBlock *)array;
+  globTab[cmsg].ops = &dataBlockSym;
+  array->value.q[0] = p_strcpy(tmsg);
+}
+
 extern VMaction BranchFalse, BranchTrue;
 
 void Y_catch(int nArgs)
@@ -1007,17 +1364,17 @@ void Y_set_idler(int nArgs)
 {
   Function *f;
   if (nArgs>1)
-    YError("set_idler function takes exactly zero or one argument");
+    YError("set_idler function takes zero or one arguments");
 
-  if (nArgs>0 && YNotNil(sp)) {
-    f= (Function *)sp->value.db;
-    if (sp->ops!=&dataBlockSym || f->ops!=&functionOps)
+  if (nArgs>0 && YNotNil(sp-nArgs+1)) {
+    f = (Function *)sp[1-nArgs].value.db;
+    if (sp[1-nArgs].ops!=&dataBlockSym || f->ops!=&functionOps)
       YError("expecting function as argument");
-    y_idler_function= Ref(f);
+    y_idler_function = Ref(f);
 
   } else if (y_idler_function) {
-    f= y_idler_function;
-    y_idler_function= 0;
+    f = y_idler_function;
+    y_idler_function = 0;
     Unref(f);
   }
 }
