@@ -1,11 +1,12 @@
 /*
-   HASH.C
-   Define interface to hash routines
-
-   $Id: hash.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: yhash.c,v 1.1.1.1 2005/09/18 22:04:09 dhmunro Exp $
+ * Define interface to hash routines
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "hash.h"
 #include "defmem.h"
