@@ -1,12 +1,13 @@
 /*
-    YRDWR.C
-    Implement YRead and YWrite functions, plus general gather/scatter
-    routines with StructDef format Converter.
-
-    $Id: yrdwr.c,v 1.1 1993/08/27 18:32:09 munro Exp $
+ * $Id: yrdwr.c,v 1.1.1.1 2005/09/18 22:04:16 dhmunro Exp $
+ * Implement YRead and YWrite functions, plus general gather/scatter
+ * routines with StructDef format Converter.
+ */
+/* Copyright (c) 2005, The Regents of the University of California.
+ * All rights reserved.
+ * This file is part of yorick (http://yorick.sourceforge.net).
+ * Read the accompanying LICENSE file for details.
  */
-/*    Copyright (c) 1994.  The Regents of the University of California.
-                    All rights reserved.  */
 
 #include "bcast.h"
 #include "pstdlib.h"
